/*
 * Created on 2004/07/30
 * Modified on 2005/08/05 by D.Fujimoto refactored the following to use WorkspaceModifyOperation:
 * 											updateGeneralSetting()
 * 											updateBuildSetting()
 * 											updateLinkerScriptSetting()
 * 											updateParameterSetting()
 * 											updateDebuggerSetting()
 * Modified on 2005/11/18 by D.Fujimoto modified createGnu33ProjectDescriptor for
 * 											initial vector section settings.
 * Modified on 2006/05/16 by D.Fujimoto added instructions in createGnu33ProjectDescriptor().
 * Modified on 2006/07/21 by D.Fujimoto removed instructions from createGnu33ProjectDescriptor() and 
 * 										added createDefaultCmdFile().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.file.CommandFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.util.Assert;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

/**
 * Provides access to Gnu33Descriptor to other classes.
 * Watch for changes to the Gnu33Project Descriptor(.gnu33project).
 * 
 * @author D.Fujimoto
 */
public class Gnu33ProjectDescriptorManager {

	private Gnu33ProjectDescriptor descriptor;
	private IProject project;
	private boolean isCreate;
	
	/**
	 * Returns the descriptor file name
	 * @return the descriptor file name
	 */
	public static String getDescriptorFileName() {
		return Gnu33ProjectDescriptor.DESCRIPTION_FILE_NAME;
	}


	/**
	 * Returns the descriptor manager for the given project.
	 * @param project	the target project
	 * @return	descriptor manager
	 * @throws CoreException 
	 * @since 1.4.1
	 */
	public static Gnu33ProjectDescriptorManager getDescriptorManager(IProject project) throws CoreException {
		// reads and returns the contents of the project descriptor file
		Gnu33ProjectDescriptorManager descriptorManager = new Gnu33ProjectDescriptorManager(project, false);
		descriptorManager.fetchProjectDescriptor();
		return descriptorManager;
	}

	/**
	 * Constructor.
	 * Only for internal use.
	 */
	protected Gnu33ProjectDescriptorManager() {
		isCreate = true;
	}
	
	/**
	 * Constructor.
	 * @param project 	Specifys the descriptors project 
	 * @param create 	True if creating a new file, false otherwise
	 * @throws CoreException
	 */
	protected Gnu33ProjectDescriptorManager(IProject project, boolean create) throws CoreException {
		setProject(project);
		isCreate = create;
	}
	
	/**
	 * Sets the targetted project.
	 * @param project
	 */
	public void setProject(IProject project) {
		this.project = project;
	}
	
	/**
	 * Creates a new file only if true was set at the constructor.
	 * @param general 	GeneralInfo object
	 * @param vector	vector section info (may be null)
	 * @throws CoreException
	 */
	public void createProjectDescriptor(GeneralInfo general, SectionData vector) throws CoreException {
		this.descriptor = new Gnu33ProjectDescriptor(project, isCreate);
		
		descriptor.prepareDefaults(general, vector);
		descriptor.createNewFile();
		
	}
	
	/**
	 * Reads the descriptor file.
	 * @throws CoreException
	 */
	public void fetchProjectDescriptor() throws CoreException {
		this.descriptor = new Gnu33ProjectDescriptor(project, isCreate); 
	}

	/**
	 * Updates the given GeneralInfo to the descriptor file.
	 * @param generalInfo
	 * @throws CoreException
	 */
	public void updateGeneralSetting(final GeneralInfo generalInfo) throws CoreException {
		// must read before updating because other settings might be modified
		WorkspaceModifyOperation op = new WorkspaceModifyOperation(project) {
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
				monitor.beginTask("Updating General Info Setting", 3);//$NON-NLS-1$
				descriptor.readFile();
				descriptor.setGeneralInfo(generalInfo);
				descriptor.createFile();
				monitor.done();
			}
		};

		try {
			op.run(new NullProgressMonitor());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw CoreException */			
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException) e.getTargetException();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw CoreException */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Updates the given BuildSettings to the descriptor file.
	 * @param buildSettingsList
	 * @throws CoreException
	 */
	public void updateBuildSetting(final List buildSettingsList) throws CoreException {
		// must read before updating because other settings might be modified
		WorkspaceModifyOperation op = new WorkspaceModifyOperation(project) {
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
				monitor.beginTask("Updating Build Setting", 3);//$NON-NLS-1$
				descriptor.readFile();
				descriptor.setBuildSettingsList(buildSettingsList);
				descriptor.createFile();
				monitor.done();
			}
		};
		
		try {
			op.run(new NullProgressMonitor());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw CoreException */			
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException) e.getTargetException();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw CoreException */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Updates the given LinkerScriptSetting to the descriptor file.
	 * @param ldsSetting
	 * @throws CoreException
	 */
	public void updateLinkerScriptSetting(final LinkerScriptSetting ldsSetting) throws CoreException {
		// must read before updating because other settings might be modified
		WorkspaceModifyOperation op = new WorkspaceModifyOperation(project) {
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
				monitor.beginTask("Updating Linkerscript Setting", 3);//$NON-NLS-1$
				descriptor.readFile();
				descriptor.setLdsSetting(ldsSetting);
				descriptor.createFile();
				monitor.done();
			}
		};
		
		try {
			op.run(new NullProgressMonitor());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw CoreException */			
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException) e.getTargetException();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw CoreException */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	
	/**
	 * Updates the given ParameterSettings to the descriptor file.
	 * @param parSettingsList
	 * @throws CoreException
	 */
	public void updateParameterSetting(final List parSettingsList) throws CoreException {
		// must read before updating because other settings might be modified
		WorkspaceModifyOperation op = new WorkspaceModifyOperation(project) {
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
				monitor.beginTask("Updating Parameter Setting", 3);//$NON-NLS-1$
				descriptor.readFile();
				descriptor.setParSettingsList(parSettingsList);
				descriptor.createFile();
				monitor.done();
			}
		};
		
		try {
			op.run(new NullProgressMonitor());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw CoreException */			
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException) e.getTargetException();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw CoreException */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Updates the given DebuggerSetting to the descriptor file.
	 * @param gdbSetting
	 * @throws CoreException
	 * @throws InterruptedException
	 * @throws InvocationTargetException
	 */
	public void updateDebuggerSetting(final DebuggerSetting gdbSetting) throws CoreException {
		// must read before updating because other settings might be modified
		WorkspaceModifyOperation op = new WorkspaceModifyOperation(project) {
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
				monitor.beginTask("Updating Debugger Setting", 3);//$NON-NLS-1$
				descriptor.readFile();
				descriptor.setGDBSetting(gdbSetting);
				descriptor.createFile();
				monitor.done();
			}
		};
		
		try {
			op.run(new NullProgressMonitor());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
/* >>>>> ADDED by D.Fujimoto 2007/08/03 throw CoreException */			
			if (e.getTargetException() instanceof CoreException) {
				throw (CoreException) e.getTargetException();
			}
/* <<<<< ADDED by D.Fujimoto 2007/08/03 throw CoreException */

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Creates a default command file from the settings in the 
	 * project descriptor.
	 * Call fetchProjectDescriptor() prior to calling this method.
	 * 
	 * @since 1.3.0
	 */
	public void createDefaultCmdFile() {

		// create a command file from the descriptor settings
		final CommandFileGenerator generator = new CommandFileGenerator(project, this, descriptor.getGDBSetting());

		try {
			generator.setContentsToStream(generator.makeContents());

			WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
				protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException {
					generator.createFile(monitor);
				}
			};
			op.run(new NullProgressMonitor());
			
			generator.releaseStream();
		
		} catch (InvocationTargetException e) {
			Gnu33CorePlugin.log(e);
		} catch (InterruptedException e) {
			Gnu33CorePlugin.log(e);
		} catch (IOException e) {
			Gnu33CorePlugin.log(e);
		}		

	}


	/**
	 * @return		the GeneralInfo contained in the descriptor file.
	 */
	public GeneralInfo getGeneralInfo() {
		Assert.isNotNull(descriptor);
		return descriptor.getGeneralInfo();
	}
	
	/**
	 * @return		the ParameterSetting List contained in the descriptor file.
	 */
	public List getParameterList() {
		Assert.isNotNull(descriptor);
		return descriptor.getParSettingsList();
	}
	
	/**
	 * @return		the BuildSetting List contained in the descriptor file.
	 */
	public List getBuildList() {
		Assert.isNotNull(descriptor);
		return descriptor.getBuildSettingsList();
	}

	/**
	 * @return		the DebuggerSetting contained in the descriptor file.
	 */
	public DebuggerSetting getGDBSetting() {
		Assert.isNotNull(descriptor);
		return descriptor.getGDBSetting();
	}
	
	/**
	 * @return		the LinkerScriptSetting contained in the descriptor file.
	 */
	public LinkerScriptSetting getLdsSetting() {
		Assert.isNotNull(descriptor);
		return descriptor.getLdsSetting();
	}
	
	/**
	 * Handy method to get the CPU type.
	 * @return		CPU type as int		
	 */
	public int getCPUType() {
		Assert.isNotNull(descriptor);
		return descriptor.getCPUType();
	}
	
	/**
	 * Returns an IFile of the descriptor file
	 * @return IFile object of the descriptor file
	 * @since 1.1.1
	 */
	public IFile getUnderlyingFile() {
		return descriptor.getFile();
	}
	
	/**
	 * Returns the version stated in the project file.
	 * This may be different from the plugin version.
	 */
	public String getVersion() {
		return descriptor.getVersion();
	}
	
	
}
