/*
 * Created on 2005/01/07
 * Modified on 2005/07/26 by D.Fujimoto changed resourceChanged() for adding operations when a 
 * 												project gets opened.
 * Modified on 2005/10/11 by D.Fujimoto changed resourceChanged() for adding operations when a
 * 												project gets renamed.
 * 										created methods scheduleOpenJobs() and scheduleRenameJobs.
 * 										refactored deleteLaunchConfig(). 
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/22 by D.Fujimoto added BuildOptionJob to scheduleOpenJobs().
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import jp.co.epson.ide.eclipse.gnu33.build.BuildOptionsUpdateJob;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * A Listener class for deleting a Project.
 * This Listener will be added at plug-in startup.
 * 
 * @author D.Fujimoto
 *
 */
public class Gnu33ProjectListener implements IResourceChangeListener {

	// projects that need modification
	private Set openedProjects = new HashSet();
	private Map renamedProjects = new HashMap();
	
	/**
	 * Constructor.
	 */
	public Gnu33ProjectListener() {
		super();
	}

	/**
	 * Checks if the Gnu33 project gets deleted or gets opened.
	 *	
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {

		// reset the Set
		openedProjects.clear();
		renamedProjects.clear();
		
		// the project handle in question
		IProject project = null;
		
		// filter by the event type(POST_CHANGE/PRE_DELETE)
		switch (event.getType()) {
			case IResourceChangeEvent.POST_CHANGE :
				// get the affected children
				IResourceDelta delta = event.getDelta();
				IResourceDelta[] childDelta = delta.getAffectedChildren();
				
				for (int i = 0; i < childDelta.length; i++) {
					IResource childRes = childDelta[i].getResource();
					int flags = childDelta[i].getFlags();
					
					// is it a project? 
					if (childRes.getType() == IResource.PROJECT) {
						project = (IProject) childRes;

						try {

							// has the project been opened?
							if ((flags & IResourceDelta.OPEN) != 0 && project.isOpen()) {
								if (project.hasNature(Gnu33ProjectNature.NATURE_ID)) {
									openedProjects.add(project);
								}
							}
						
							// has the project been renamed?
							if ((flags & IResourceDelta.MOVED_FROM) != 0) {
								if (project.hasNature(Gnu33ProjectNature.NATURE_ID)) {
									String prevName = childDelta[i].getMovedFromPath().lastSegment();
									renamedProjects.put(project, prevName);
								}
							}
						
						} catch (CoreException e) {
							Gnu33CorePlugin.log(
									new Status(IStatus.WARNING, Gnu33CorePlugin.getUniqueIdentifier(), IStatus.WARNING,
											ResourceProvider.getResourceString("nature.failed"), null));
						}

					}
										
				}

				scheduleOpenJobs(openedProjects);
				scheduleRenameJobs(renamedProjects);
				
				break;
				
			case IResourceChangeEvent.PRE_DELETE :
				if (event.getResource() != null && event.getResource().getType() == IResource.PROJECT) {
					project = (IProject) event.getResource();
					try {
						if (project.hasNature(Gnu33ProjectNature.NATURE_ID)) {
							// delete the Launch Configuration
							Gnu33ProjectFactory.deleteLaunchConfig(project);	
						}
					} catch (CoreException e) {
						Gnu33CorePlugin.log(
								new Status(IStatus.WARNING, Gnu33CorePlugin.getUniqueIdentifier(), IStatus.WARNING,
										ResourceProvider.getResourceString("nature.failed"), null));
					}
				}
				break;
				
			default :
				break;
		}
		
	}

	/**
	 * Schedules some jobs for opened projects.
	 * 
	 * @param projects		the target projects
	 * @since 1.2.0
	 */
	private void scheduleOpenJobs(Set projects) {
		
		for (Iterator iter = projects.iterator(); iter.hasNext();) {
			IProject proj = (IProject) iter.next();
			Job job = new ProjectFilePathUpdater(proj);
			job.schedule();

/* >>>>> ADDED by D.Fujimoto 2007/08/22 update build options */			
			Job buildUpdateJob = new BuildOptionsUpdateJob(proj);
			buildUpdateJob.schedule();
/* <<<<< ADDED by D.Fujimoto 2007/08/22 update build options */
		}
	
	}
	
	/**
	 * Schedules some jobs for renamed projects.
	 * 
	 * @param map		the target projects
	 * @since 1.2.0
	 */
	private void scheduleRenameJobs(Map map) {
		
		Set keys = map.keySet();
		for (Iterator iter = keys.iterator(); iter.hasNext(); ) {
			IProject proj = (IProject) iter.next();
			Job job = new ProjectRenameJob(proj, (String) map.get(proj));
			job.schedule();
		}

	}
	
}
