/*
 * Created on 2005/05/25
 * Modified on 2005/12/02 by D.Fujimoto moved isValidTranslationUnit() to ProjectAccessor.
 * Modified on 2006/01/23 by D.Fujimoto modified lookupEvent() and visitor's
 * 											processRemoved() for special treatment when removing
 * 											source files. 
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/05/25 by D.Fujimoto deleting source folders did not update project file.
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import jp.co.epson.ide.eclipse.gnu33.build.DeleteDependencyFilesJob;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptUpdater;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.jobs.Job;

/**
 * This class Listens to source file change events and 
 * updates the linkerscript setting to project file.
 * @author D.Fujimoto
 * @since 1.1.1
 *  
 */
public class SourceUnitListener implements IResourceChangeListener {

	// the file in question
	private IFile file;

	// holds the affected project names
	private Set changedProjects = new HashSet();

	// holds the deleted/moved source files
	private Set removedSourceFiles = new HashSet();
	
	/*
	 * ( Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {

		// clear the Set and check what changes have been made
		changedProjects.clear();
		removedSourceFiles.clear();
		lookupEvent(event);

	}

	/**
	 * Capture POST_CHANGE events and travel the ResourceDelta
	 * 
	 * @param e
	 *            IResourceChangeEvent
	 */
	private void lookupEvent(IResourceChangeEvent e) {

		try {
			// Listen to POST_CHANGE events
			if (e.getType() == IResourceChangeEvent.POST_CHANGE) {
				if (Gnu33CorePlugin.isDebug) {
					System.out.println("Resources have changed.");
				}

				// visit the delta and gather the affected project names
				e.getDelta().accept(new SourceUnitVisitor());

				// update the linkerscript setting for the changed projects
				// if a project is considered changed, then the project file will
				// always be modified
				for (Iterator iter = changedProjects.iterator(); iter.hasNext();) {
					String projectName = (String) iter.next();
					Job job = new LinkerScriptUpdater(projectName);
					job.schedule();
				}
				
				// erase associated files that go with the removed source files
				for (Iterator iter = removedSourceFiles.iterator(); iter.hasNext();) {
					IFile removed = (IFile) iter.next();
					Job job = new DeleteDependencyFilesJob(removed);
					job.schedule();
				}
			}
		} catch (CoreException e1) {
			e1.printStackTrace();
		}
	}


	/**
	 * The ResourceDeltaVisitor for checking what files have been added to the
	 * Project.
	 * 
	 * @author D.Fujimoto
	 *  
	 */
	private class SourceUnitVisitor implements IResourceDeltaVisitor {

		public boolean visit(IResourceDelta delta) throws CoreException {

			IResource res = delta.getResource();

			// process only FILEs
			if (res.getType() == IResource.FILE) {
				file = (IFile) res;

				// check for only source files in a GNU33 project
				if (ProjectAccessor.isValidTranslationUnit(file)) {

					// visit the delta
					switch (delta.getKind()) {
						case IResourceDelta.ADDED:
							processAdded(file, delta);
							break;

						case IResourceDelta.REMOVED:
							processRemoved(file, delta);
							break;

//						case IResourceDelta.CHANGED:
//							processChanged(file, delta);
//							break;

					}
					
/* >>>>> MODIFIED by D.Fujimoto 2007/05/25 deleting source folders did not update project file */
				} else 	if (isSourceFolderChanged(delta)) {
					if (Gnu33CorePlugin.isDebug) {
						System.out.println("Assuming source folder changed");
					}

					// preferable to run the job at a low priority to avoid dead locks
					changedProjects.add(res.getProject().getName());

/* <<<<< MODIFIED by D.Fujimoto 2007/05/25 deleting source folders did not update project file */	
				}
				
				
				return false;
			}

			return true; // visit the children
		}

		
		/**
		 * Returns wheter a source folder has been set or removed to the project.
		 * This method exists because we want to update the linker script settings
		 * whenever a source folder changes (which we won't be reported via a resource change event) 
		 *   
		 * @param delta	the ResourceDelta to examine
		 * @return	true if the source folder has been set/removed
		 * @since 1.4.0
		 * ADDED by D.Fujimoto 2007/05/25
		 */
		private boolean isSourceFolderChanged(IResourceDelta delta) {
			boolean changed = false;
			
			IResource res = delta.getResource();
	
			// in order to determine that a source folder has been
			// added/removed, we will see changes to the CDT project descriptor
			if (res.getType() == IResource.FILE) {
				IFile file = (IFile) res;
				
				if (file.getName().equals(".cdtproject") && 
						((delta.getFlags() & IResourceDelta.CONTENT) != 0)) {
					changed = true;
				}
				
			}
			
			return changed;
			
		}
		
		
		/**
		 * Check IResourceDelta.ADDED modifications. Set the project name to the
		 * Set
		 * 
		 * @param addedFile
		 * @param delta
		 */
		private void processAdded(IFile addedFile, IResourceDelta delta) {
			int flags = delta.getFlags();
			if (Gnu33CorePlugin.isDebug) {
				System.out.println("->Resource " + file.getFullPath()
						+ " was ADDED.");
			}

			// if it is not added to a source folder, do nothing
			if (!ProjectAccessor.isContainedInSourceRoot(file)) {
				return;
			}

			if ((flags & IResourceDelta.MOVED_FROM) != 0) {
				// moved or renamed
				changedProjects.add(file.getProject().getName());
			} else {
				// simply added
				changedProjects.add(file.getProject().getName());
			}

		}

		/**
		 * Check IResourceDelta.REMOVED modifications. Set the project name to
		 * the Set
		 * 
		 * @param removedFile
		 * @param delta
		 */
		private void processRemoved(IFile removedFile, IResourceDelta delta) {
			int flags = delta.getFlags();
			if (Gnu33CorePlugin.isDebug) {
				System.out.println("->Resource " + file.getFullPath()
						+ " was REMOVED.");
			}

/* >>>>> MODIFIED by D.Fujimoto 2007/05/25 deleting source folders did not update project file */
			// if it is not added to a source folder, 
			// and if was not a source, do nothing
			if (!ProjectAccessor.isContainedInSourceRoot(file) &&
					!ProjectAccessor.isValidTranslationUnit(file)) {
				return;
			}
/* <<<<< MODIFIED by D.Fujimoto 2007/05/25 deleting source folders did not update project file */
			
			// moved to another folder
			if ((flags & IResourceDelta.MOVED_TO) != 0) {
				changedProjects.add(file.getProject().getName());
				removedSourceFiles.add(file);
			} else {
				// simply removed
				changedProjects.add(file.getProject().getName());
				removedSourceFiles.add(file);
			}
		}

		/**
		 * Check IResourceDelta.CHANGED modifications.
		 * 
		 * @param changedFile
		 * @param delta
		 */
		private void processChanged(IFile changedFile, IResourceDelta delta) {
			int flags = delta.getFlags();
			if (Gnu33CorePlugin.isDebug) {
				System.out.println("->Resource " + file.getFullPath()
						+ " was CHANGED.");
			}
		}

		/**
		 * A helper method that displays the Delta flag
		 * 
		 * @param flags
		 *            IResourceDelta.getFlags()
		 */
		private void compFlags(int flags) {

			if ((flags & IResourceDelta.REPLACED) != 0) {
				System.out.println("\tand replaced");
			}
			if ((flags & IResourceDelta.MOVED_FROM) != 0) {
				System.out.println("\tand moved from");
			}
			if ((flags & IResourceDelta.MOVED_TO) != 0) {
				System.out.println("\tand moved to");
			}

		}

	}

}
