/*
 * Created on 2004/08/17
 * Modified on 2005/10/03 by D.Fujimoto changed if statement in getEnumCommand()
 * 											because of BUG. 
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.cdt.managedbuilder.core.IOption;

/**
 * An implementation of IBuildOption.
 * Represents a single option of a tool.
 * @author D.Fujimoto
 *
 */
public class BuildOption implements IBuildOption {

	private static final String EMPTY_STRING = new String();
	private static final String[] EMPTY_STRING_ARRAY = new String[0];
	private static final int DEFAULT_ENUM_VALUE = 0;
	
	
	/**
	 * The id for this option.
	 * Also used for description file.
	 */
	private String id;
	
	/**
	 * The type of this option.
	 * @see IBuildOption
	 */
	private int type;	 
	
	/**
	 * Command of the option
	 * In use when type is TYPE_STRING | TYPE_BOOLEAN | TYPE_LIST
	 */
	private String command;
	
	/**
	 * Displayable String for the option.
	 * Used as the UI label.
	 */
	private String label;
	
	/**
	 * The category for this option.
	 */
	private IBuildCategory category;
	
	/**
	 * List for command and label
	 * Command and label will be set as a 2 length String array
	 * In use when type is TYPE_ENUM 
	 */
	private List commandList;
	
	/**
	 * The command when this option is selected as "false"
	 * Used only when type is TYPE_BOOLEAN
	 */
	private String commandFalse;
	
   
	/**
	 * true if this option requires arguments
	 */
	private boolean needArgs;
	
  
	/**
	 * The list type for creating a List control
	 * Possible values are, BROWSE_NONE | BROWSE_FILE | BROWSE_DIR
	 * derived from org.eclipse.cdt.managedbuilder.core.IOption
	 *	
	 * In use when type is TYPE_LIST
	 */
	private int browseType = IOption.BROWSE_NONE;
	
	/**
	 * The value of the option is stored here.
	 * Object type depends on the type.
	 * <ul>
	 * <li>For TYPE_BOOLEAN, it is stored as Boolean. 
	 * 		Values are set with setBooleanValue()</li>
	 * <li>For TYPE_STRING and TYPE_STRING_MULTI, it is stored as String.
	 * 		Values are set with setStringValue()</li>
	 * <li>For TYPE_LIST, it is stored as List of String[].
	 * 		Values are set with setListValue()</li>
	 * <li>For TYPE_ENUM, it is stored as Integer(of the Combo's index).
	 * 		Values are set with setEnumValue()</li>
	 */
	private Object value;
	
	/**
	 * Flag that makes this option invisible in the UI.
	 */
	private boolean invisible;
	
	/**
	 * Constructor.
	 * Creates a new BuildOption.
	 * @param id		ID for the option, also used to write to descriptor file
	 * @param command	The command line String
	 * @param type		Option type
	 * @param category	Category which this option belongs to
	 */
	public BuildOption(String id, String command, int type, IBuildCategory category) {
		super();
		
		this.category = category;	   
		category.addOption(this);
		
		setId(id);
		setCommand(command);
		setType(type);
		switch (type) {
			case TYPE_BOOLEAN :
				setBooleanValue(false);
				break;
			case TYPE_ENUM :
				setEnumValue(DEFAULT_ENUM_VALUE);
				break;
			case TYPE_LIST :
				setListValue(EMPTY_STRING_ARRAY);
				break;
			case TYPE_STRING :
			case TYPE_STRING_MULTI :
				setStringValue(EMPTY_STRING);
				break;
		}

	}

	/**
	 * Returns the tool for this option.
	 */
	public IBuildTool getTool() {
		return category.getTool();
		
	}
	
	/**
	 * Returns the category for this option.
	 */
	public IBuildCategory getCategory() {
		return category;
	}

	// getter/setter
	public String getId() {
		return id;
	}
	public void setId(String id) {
		this.id = id;
	}
	public boolean isNeedArgs() {
		return needArgs;
	}
	public void setNeedArgs(boolean needArgs) {
		this.needArgs = needArgs;
	}
	public String getCommand() {
		return command;
	}
	public void setCommand(String command) {
		this.command = command;
	}
	public String getLabel() {
		return label;
	}
	public void setLabel(String label) {
		this.label = label;
	}
	public int getType() {
		return type;
	}
	public void setType(int type) {
		this.type = type;
	}
	public String getCommandFalse() {
		return commandFalse;
	}
	public void setCommandFalse(String commandFalse) {
		this.commandFalse = commandFalse;
	}
	public List getCommandList() {
		return commandList;
	}
	public void setCommandList(List commandList) {
		this.commandList = commandList;
	}
	public int getBrowseType() {
		return browseType;
	}
	public void setBrowseType(int browseType) {
		this.browseType = browseType;
	}
	public void setInvisible(boolean flag) {
		invisible = flag;
	}
	public boolean isInvisible() {
		return invisible;
	}

	/**
	 * Returns the label for the Combos.
	 */
	public String[] getEnumOptions() {
		String[] ret = null;
		if (commandList.size() > 0) {
			ret = new String[commandList.size()];
			
			for (int i = 0; i < ret.length; i++) {
				String[] commandAndDisplay = (String[]) commandList.get(i);
				ret[i] = commandAndDisplay[1];
			}
			return ret;
		}
		return new String[0];
	}
 
	/**
	 * Returns the command for responding the index stored in BuildOption#value.
	 * To use this method, the type must be TYPE_ENUM.
	 */
	public String getEnumCommand() {
		int selectedIndex = getEnumValue();
		
		if (selectedIndex < commandList.size()) {
			String[] commandAndDisplay = (String[]) commandList.get(selectedIndex);
			return commandAndDisplay[0];
		} else {
			String[] commandAndDisplay = (String[]) commandList.get(0);
			return commandAndDisplay[0];
		}
	}
	
	/**
	 * Returns the command for the selected Combo.
	 * @param selectedValue the label of which the Combo is selected 
	 */
	public String getEnumCommand(String selectedValue) {
		String[] enums = getEnumOptions();
		for (int i = 0; i < enums.length; i++) {
			if (selectedValue.equals(enums[i])) {
				String[] commandAndDisplay = (String[]) commandList.get(i);
				return commandAndDisplay[0];
			}
		}
		
		return null;
	}
	
  

	/** value accessors ***/
	/**
	 * Returns the value of this option.
	 * @return true if this option is selected
	 */
	public boolean getBooleanValue() {
		if (type != TYPE_BOOLEAN) {
			return false;
		}
		
		Boolean bool = (Boolean) value;
		return bool.booleanValue();
	}
	
	/**
	 * Sets the value of this option.
	 * @param value		true if this option should be selected
	 */
	public void setBooleanValue(boolean value) {
		if (type != TYPE_BOOLEAN) {
			// error
		}
		this.value = Boolean.valueOf(value);
	}
	
	/**
	 * Returns the value of this option.
	 * @return the option in String
	 */
	public String getStringValue() {
		if (type != TYPE_STRING && type != TYPE_STRING_MULTI) {
			// error
		}
		return value == null ? EMPTY_STRING : (String) value;
	}
	
	/**
	 * Sets the value of this option.
	 * @param value		the value in String
	 */
	public void setStringValue(String value) {
		if (type != TYPE_STRING && type != TYPE_STRING_MULTI) {
			// error
		}
		this.value = value;
	}
	
	/**
	 * Returns the values of this option.
	 * @return the list values as String[]
	 */
	public String[] getListValue() {
		if (type != TYPE_LIST) {
			// error
		}
		
		List list = (List) value;
		if (list == null) {
			return EMPTY_STRING_ARRAY;
		} else {
			return (String[]) list.toArray(new String[0]);
		}
	}
	
	/**
	 * Sets the values of this option.
	 * @param values	the list values as String[] 
	 */
	public void setListValue(String[] values) {
		if (type != TYPE_LIST) {
			// error
		}
		List list = new ArrayList();
		for (int i = 0; i < values.length; i++) {
			list.add(values[i]);
		}
		
		value = list;
	}

	/**
	 * Returns the value of this option.
	 * @return the enum index based on 0
	 */
	public int getEnumValue() {
		if (type != TYPE_ENUM) {
			// error
		}
		return value == null ? DEFAULT_ENUM_VALUE : ((Integer) value).intValue();
		
	}

	/**
	 * Sets the value of this option.
	 * @param value		the enum index based on 0
	 */
	public void setEnumValue(int value) {
		if (type != TYPE_ENUM) {
			// error
		}
		this.value = new Integer(value); 
		
	}

	public String toString() {
		return id + "=" + value; //$NON-NLS-1$
	}

}
