/*
 * Created on 2007/08/22
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Util class that updates the project file as a Job.
 * This class will modify 
 * user options of the compiler and assembler which resides in
 * the .project file.
 * 
 * @author D.Fujimoto
 * @since 1.4.0
 * @see jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectListener
 *
 */
public class BuildOptionsUpdateJob extends WorkspaceJob {

	public static final String JOB_NAME = "project file build options updater";
	
	/**
	 * The project subject to update
	 */
	private IProject project;
	
	/**
	 * The family name of this job
	 */
	protected String family;
	
	
	/**
	 * @param projectName	
	 * 			the project which linkerscript setting needs to be changed
	 */
	public BuildOptionsUpdateJob(String projectName) {
		super(JOB_NAME);
		this.family = projectName;
	}
	
	/**
	 * 
	 * @param project	the target project.
	 */
	public BuildOptionsUpdateJob(IProject project) {
		this(project.getName());
		this.project = project; 

		setPriority(Job.SHORT);
		setSystem(true);
		// the scheduling rule should be the workspace root
		setRule(ResourcesPlugin.getWorkspace().getRoot());
	
	}

	/* ( Javadoc)
	 * @see org.eclipse.core.resources.WorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor)
			throws CoreException {

		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		try {
			monitor.beginTask("Updating " + IProjectDescription.DESCRIPTION_FILE_NAME + " for " + project.getName(), 1);
			
			// update settings
			updateUserOptions(project);
			monitor.worked(1);

			monitor.done();
			
		} catch (CoreException e) {
			e.printStackTrace();
			Logger.log("Failed to update " + IProjectDescription.DESCRIPTION_FILE_NAME + " for " + project.getName());
			return e.getStatus();
		}
		return Status.OK_STATUS;
	}

	
	/**
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#belongsTo(java.lang.Object)
	 */
	public boolean belongsTo(Object family) {
		return this.family.equals(family);
	}

	
	/**
	 * Rewrites the User Options in the build settings. 
	 * @param project
	 */
	public synchronized void updateUserOptions(IProject project) throws CoreException {
		
		Gnu33ProjectDescriptorManager man = Gnu33ProjectDescriptorManager.getDescriptorManager(project);

		int cpuType = man.getCPUType();
		List settingList = man.getBuildList();
		BuildSetting buildSetting = (BuildSetting) settingList.get(0);

		
		if (BuildSettingFactory.shouldAlterMiscOptions(buildSetting)) {
			
			// delete cpu options from misc
			BuildSettingFactory.switchCpuOptions(buildSetting, cpuType);
			
			man.updateBuildSetting(settingList);
		}		
		
	}
	
	
	/**
	 * The job will be scheduled if the target project exists and is open.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldSchedule()
	 */
	public boolean shouldSchedule() {
		if (!project.exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}
	
	/**
	 * The job will be run if the target project exists.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		if (!project.exists() || !project.getFile(IProjectDescription.DESCRIPTION_FILE_NAME).exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}

}
