/*
 * Migrated for GNU33 on 2007/01/19 from GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.cpuinfo;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Comparator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.SortedMap;
import java.util.StringTokenizer;
import java.util.TreeMap;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptor;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterArea;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Manager for CPU specific default settings
 * @author D.Fujimoto
 * @since 1.4.0
 *
 */
public class CpuSettingsManager implements ICpuPreferenceConstants {

	// cpu files directory as a relative path from eclipse.exe
//	private static final String REL_PATH = "/workspace/jp.co.epson.ide.eclipse.gnu33/cpuinfo";	// debug
	private static final String REL_PATH = "/cpuinfo";

	private static final String FILE_PREFIX = "S1C33";
	private static final String FILE_SUFFIX = ".properties";

	private static final String INITIAL_VERSION = "1.4.0";
	
	// 3 default CPU names(files)
	public static final String STD_NAME = FILE_PREFIX + GeneralInfo.CPU_STD_STR;
	public static final String PE_NAME = FILE_PREFIX + GeneralInfo.CPU_PE_STR;
	public static final String ADV_NAME = GeneralInfo.CPU_ADV_33401_STR;
	
	/**
	 * the singleton instance
	 */
	private static CpuSettingsManager MANAGER;

	/**
	 * Initialized flag
	 */
	private boolean initialized;
	
	/**
	 * The directory that contains the files
	 */
	private File cpuDir;
	
	/**
	 * The sorted map of GeneralInfo created by reading each cpu file
	 * Key is CPU name, value is GeneralInfo object
	 * This map will hold the cpu names and will be the key to
	 * obtain their further settings.
	 */
	private SortedMap infoMap = new TreeMap(new CpuNameComparator());

	/**
	 * The map of the cpu name(key) and location path of
	 * their corresponding setting files directory(value)
	 * Key is the CPU name, value is IPath for the path
	 * @since 1.4.1
	 */
	private SortedMap cpuDirMap = new TreeMap();

	/**
	 * Returns the single manager
	 * @return	singleton manager
	 */
	public synchronized static CpuSettingsManager getManager() {
		if (MANAGER == null) {
			MANAGER = new CpuSettingsManager();
//			MANAGER.initialize(ToolPathVarResolver.getGnuToolsPath());
			MANAGER.initialize(Gnu33CorePlugin.getPlatformInstallPath());	// 33 folder is in /eclipse
		}
		return MANAGER;
	}
	
	
	/**
	 * Initializes the IPreferenceStore with default values.
	 * Called by the PreferenceInitializer.
	 * @see <code>initializeStoreByCpu()</code>
	 */
	public static void initializeStoreWithDefault(IPreferenceStore store) {
		
		// default values for project data
		// GeneralInfo defaults to the constructor value
		store.setDefault(CPU_TYPE, GeneralInfo.CPU_STD);
		store.setDefault(CPU_NAME, "unknown");
/* >>>>> ADDED by D.Fujimoto 2008/05/30 added display name */
		store.setDefault(CPU_DISP_NAME, "unknown");
/* <<<<< ADDED by D.Fujimoto 2008/05/30 added display name */
		store.setDefault(MEM_MODEL, GeneralInfo.MEM_32);
		store.setDefault(VECTOR_ADDRESS, GeneralInfo.TTBR_DEFAULT);
		store.setDefault(HIGH_LIB, false);

				
		// build options (see BuildSettingFactory)
		// note that if you add or delete the keys, do the same
		// in setBuildSettingsToStore()
		store.setDefault(BuildSettingFactory.OPT_LEVEL_OPT_ID, 1);			// selection index
		store.setDefault(BuildSettingFactory.NO_BUILTIN_OPT_ID, true);
		store.setDefault(BuildSettingFactory.NO_EXPAND_MEMCPY_OPT_ID, false);
		store.setDefault(BuildSettingFactory.FPIC_OPT_ID, false);
		store.setDefault(BuildSettingFactory.LONG_CALL_OPT_ID, true);
		store.setDefault(BuildSettingFactory.DEFINE_OPT_ID, "");			// must be comma separated
		store.setDefault(BuildSettingFactory.C_USER_OPT_ID, "");			// will be appended 
		
		store.setDefault(BuildSettingFactory.ASM_USER_OPT_ID, "");			// will be appended

		store.setDefault(BuildSettingFactory.GEN_MAP_OPT_ID, true);
		store.setDefault(BuildSettingFactory.NO_ALIGN_CHECK_OPT_ID, true);
		store.setDefault(BuildSettingFactory.LD_USER_OPT_ID, "");			// will be appended
		
		store.setDefault(Gnu33ProjectDescriptor.TAG_ENV_VAR, "");			// semicolon separated
		
		// linkerscript settings
		store.setDefault(BSS_VMA, 0x0L);
		
		
		// parameter area settings
		store.setDefault(PAR_AREA_COUNT, 2);
		
		store.setDefault(PAR_AREA_TYPE + "0", ParameterArea.AREA_RAM);
		store.setDefault(PAR_AREA_START + "0", 0x0L);
		store.setDefault(PAR_AREA_SIZE + "0", 0x1000000L);
		store.setDefault(PAR_AREA_ACCESS_SIZE + "0", ParameterArea.ACCESS_WORD);
		store.setDefault(PAR_AREA_READ_WAIT + "0", 0);
		store.setDefault(PAR_AREA_WRITE_WAIT + "0", 0);
		store.setDefault(PAR_BIG_ENDIAN + "0", false);
		store.setDefault(PAR_AREA_COMMENT + "0", "");
		
		store.setDefault(PAR_AREA_TYPE + "1", ParameterArea.AREA_STACK);
		store.setDefault(PAR_AREA_START + "1", 0x0L);
		store.setDefault(PAR_AREA_SIZE + "1", 0x4000L);
		store.setDefault(PAR_AREA_ACCESS_SIZE + "1", ParameterArea.ACCESS_WORD);
		store.setDefault(PAR_AREA_READ_WAIT + "1", 0);
		store.setDefault(PAR_AREA_WRITE_WAIT + "1", 0);
		store.setDefault(PAR_BIG_ENDIAN + "1", false);
		store.setDefault(PAR_AREA_COMMENT + "1", "");
		
		
		// command settings
		store.setDefault(DEBUGGER_TYPE, DebuggerSetting.TYPE_DEFAULT);
		store.setDefault(DEBUG_RAM_ADDR, DebuggerSetting.RAM_ADDR_DEFAULT);
		
	}
	
	
	/**
	 * Private constructor.
	 */
	private CpuSettingsManager() {}
	
	
	/**
	 * Initializes this manager i.e, reads the setting file and
	 * makes a Map of GeneralInfo objects.
	 * Field cpuDir will be set in this method.
	 * @param	path	full location to the file directory
	 */
	public boolean initialize(IPath path) {
		initialized = false;
		
		IPath filePath = path.append(REL_PATH);
		
		try {
			// check the file existence
			cpuDir = filePath.toFile();

			// make the Map from reading the files
			if (cpuDir.exists() && cpuDir.canRead() && cpuDir.isDirectory()) {
				makeCpuInfoMap();
			}
			if (infoMap.isEmpty()) {
				throw new IOException();
			}
			initialized = true;
			
		} catch (IOException e) {
			// should never occur
			e.printStackTrace();
			Logger.log("Could not initialize cpu info");
		}
		
		return initialized; 
	}
	
	
	/**
	 * Reads the cpu settings(only general info) from a all files in the given directory
	 * and creates a Map of GeneralInfo objects used for selecting in the 
	 * project wizard.
	 * Key=Value -> CPU name=GeneralInfo object.
	 * 
	 */
	private void makeCpuInfoMap() throws IOException {

		// get the file as List
		File[] fileList = getFileList();
		
		// read them as property files
		Properties cpuProps = new Properties();
		FileInputStream stream = null;
		for (int i = 0; i < fileList.length; i++) {
			stream = new FileInputStream(fileList[i]);
			cpuProps.load(stream);
			stream.close();
			
			// get the cpu info(default from store)
			String cpuType = cpuProps.getProperty(CPU_TYPE, GeneralInfo.CPU_STD_STR);
			String cpuName = cpuProps.getProperty(CPU_NAME, GeneralInfo.CPU_STD_STR);
/* >>>>> ADDED by D.Fujimoto 2008/05/30 added cpu display name */
			String cpuDisplayName = cpuProps.getProperty(CPU_DISP_NAME, cpuName);
/* <<<<< ADDED by D.Fujimoto 2008/05/30 added cpu display name */
			String memoryModel = cpuProps.getProperty(MEM_MODEL, GeneralInfo.MEM_32_STR);
			String[] configs = splitValues(cpuProps.getProperty("configs", "default"), ",");
			String ttbrStr = cpuProps.getProperty(VECTOR_ADDRESS);
			String useLib = cpuProps.getProperty(HIGH_LIB, "false");
			
			// create a GeneralInfo object from the file contents
			// this object will be used in the project wizard
			GeneralInfo general = new GeneralInfo();
			general.setCpuType(GeneralInfo.parseCPUInt(cpuType));
			general.setCpuName(cpuName);
/* >>>>> ADDED by D.Fujimoto 2008/05/30 added cpu display name */
			general.setCpuDisplayName(cpuDisplayName);
/* <<<<< ADDED by D.Fujimoto 2008/05/30 added cpu display name */
			general.setMemoryModel(GeneralInfo.parseMemModelInt(memoryModel));
			general.setCurrentConfName(configs[0]);
			if (ttbrStr != null && ttbrStr.length() != 0) {
				general.setInitialTtbrValue(Long.parseLong(ttbrStr, 16));
			}
			general.setUseHighPrecisionLib(Boolean.valueOf(useLib).booleanValue());
			
			// add it to the map
			infoMap.put(cpuName, general);

			
/* >>>>> ADDED by D.Fujimoto 2008/05/30 store mappings for CPU directory */
			createCpuPathMap(cpuName, fileList[i]);
/* <<<<< ADDED by D.Fujimoto 2008/05/30 store mappings for CPU directory */

		}
	}	
	
	/**
	 * Creates the CPU path map
	 * @param 	cpuName		the target CPU
	 * @param	filePath	File object for the CPU file
	 * @since 1.4.1
	 */
	private void createCpuPathMap(String cpuName, File filePath) {
		// get the absolute path as IPath
		IPath path = new Path(filePath.getParentFile().getAbsolutePath());

		// add it to the map
		cpuDirMap.put(cpuName, path);
	}

	/**
	 * Initializes the PreferenceStore from the given cpu settings declared in GeneralInfo
	 * convinience method.
	 * @param store		the IPreferenceStore to initialize
	 * @param general		the cpu info and file to be read is in this object
	 * @throws IOException
	 */
	protected void initializeStoreByCpu(IPreferenceStore store, GeneralInfo general) throws IOException {
		String cpuName = general.getCpuName();
		initializeStoreByCpu(store, cpuName);
	}
	
	/**
	 * Initializes the PreferenceStore from the given cpu settings declared in GeneralInfo
	 * This is typically called when the project wizard is complete.
	 * 
	 * @param store		the IPreferenceStore to initialize
	 * @param cpuName		the cpu in question
	 * @throws IOException
	 */
	public void initializeStoreByCpu(IPreferenceStore store, String cpuName) throws IOException {
		
		// the cpu's file
		File cpuFile = null;
		
		// get the file as List
		File[] fileList = getFileList();
		for (int i = 0; i < fileList.length; i++) {
			if (fileList[i].getName().equals(cpuName + FILE_SUFFIX)) {
				cpuFile = fileList[i];
				break;
			}
		}

		// throw an Exception when no file was found
		if (cpuFile == null) {
			String msg = MessageFormat.format(ResourceProvider.getResourceString("msg.cpuFileNotFound"), new Object[]{ cpuName });
			throw new FileNotFoundException(msg);
		}
		
		// load the file
		Properties cpuProp = new Properties();
		FileInputStream stream = new FileInputStream(cpuFile);
		cpuProp.load(stream);
		stream.close();
		
		// get the values and load them to the store
		try {

			// The filled Store will be used in createDefault()
//			String cpuType = cpuProp.getProperty(CPU_TYPE);		
//			System.out.println(cpuName + "=" + cpuType);
			
			setBuildSettingsToStore(cpuProp, store);

			setLdsSettingsToStore(cpuProp, store);
			
			setParSettingsToStore(cpuProp, store);
			
			setCmdSettingsToStore(cpuProp, store);
			
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("Problem in parsing " + cpuFile.getName());
		}

	}


	/**
	 * Reads the Build settings from the properties file and store it into the given IPreferenceStore.
	 * @param cpuProp	the properties file to read from
	 * @param store	the IPreferenceStore to write to
	 */
	private void setBuildSettingsToStore(Properties cpuProp, IPreferenceStore store) throws Exception {
		// integer options
		String intOpt = null;
		String[] intOptKeys = {
				BuildSettingFactory.OPT_LEVEL_OPT_ID
		};
		for (int i = 0; i < intOptKeys.length; i++) {
			intOpt = cpuProp.getProperty(intOptKeys[i]);
			if (intOpt != null) {
				store.setValue(intOptKeys[i], Integer.parseInt(intOpt));
			} else {
				store.setToDefault(intOptKeys[i]);
			}
		}
		
				
		// boolean options
		String boolOpt = null;
		String[] boolOptKeys = {
				BuildSettingFactory.NO_BUILTIN_OPT_ID, 
				BuildSettingFactory.NO_EXPAND_MEMCPY_OPT_ID,
				BuildSettingFactory.FPIC_OPT_ID,
				BuildSettingFactory.LONG_CALL_OPT_ID,
				BuildSettingFactory.GEN_MAP_OPT_ID, 
				BuildSettingFactory.NO_ALIGN_CHECK_OPT_ID
		};
		for (int i = 0; i < boolOptKeys.length; i++) {
			boolOpt = cpuProp.getProperty(boolOptKeys[i]);
			if (boolOpt != null) {
				store.setValue(boolOptKeys[i], Boolean.valueOf(boolOpt).booleanValue());
			} else {
				store.setToDefault(boolOptKeys[i]);
			}
		}
		
		// string options
		String strOpt = null;
		String[] strOptKeys = {
				BuildSettingFactory.DEFINE_OPT_ID, 
				BuildSettingFactory.C_USER_OPT_ID,
				BuildSettingFactory.ASM_USER_OPT_ID,
				BuildSettingFactory.LD_USER_OPT_ID,
				Gnu33ProjectDescriptor.TAG_ENV_VAR
		};
		for (int i = 0; i < strOptKeys.length; i++) {
			strOpt = cpuProp.getProperty(strOptKeys[i]);
			if (strOpt != null) {
				store.setValue(strOptKeys[i], strOpt);
			} else {
				store.setToDefault(strOptKeys[i]);
			}
		}
	}
	
	
	/**
	 * Reads the Linkerscript settings from the properties file and store it into the given IPreferenceStore.
	 * @param cpuProp	the properties file to read from
	 * @param store	the IPreferenceStore to write to
	 * @throws Exception
	 */
	private void setLdsSettingsToStore(Properties cpuProp, IPreferenceStore store) throws Exception {

		// just load the addresses for .bss(RAM) section
		String bssVma = cpuProp.getProperty(BSS_VMA);
		try {
			store.setValue(BSS_VMA, Long.parseLong(bssVma, 16));
		} catch (NumberFormatException e) {
			store.setToDefault(BSS_VMA);
		}
		
	}
	
	
	/**
	 * Reads the Parameter settings from the properties file and stores it into the given IPreferenceStore.
	 * @param cpuProp	the properties file to read from
	 * @param store	the IPreferenceStore to write to
	 * @throws Exception
	 */
	private void setParSettingsToStore(Properties cpuProp, IPreferenceStore store) throws Exception {
		// parameter area count
		String areaCount = cpuProp.getProperty(PAR_AREA_COUNT);
		try {
			store.setValue(PAR_AREA_COUNT, Integer.parseInt(areaCount));
		} catch (NumberFormatException e) {
			store.setToDefault(PAR_AREA_COUNT);
		}
		
		// get each parameter setting
		for (int i = 0; i < store.getInt(PAR_AREA_COUNT); i++) {
			String typeStr = cpuProp.getProperty(PAR_AREA_TYPE + i);
			int type = ParameterArea.parseAreaTypeInt(typeStr);
			store.setValue(PAR_AREA_TYPE + i, type);
			
			String startAddrStr = cpuProp.getProperty(PAR_AREA_START + i);
			long startAddr = Long.parseLong(startAddrStr, 16);
			store.setValue(PAR_AREA_START + i, startAddr);
			
			String sizeStr = cpuProp.getProperty(PAR_AREA_SIZE + i);
			long size = Long.parseLong(sizeStr, 16);
			store.setValue(PAR_AREA_SIZE + i, size);
			
			// no need to read when area is STACK
			if (type != ParameterArea.AREA_STACK) {
				String accessSize = cpuProp.getProperty(PAR_AREA_ACCESS_SIZE + i);
				if (accessSize != null) {
					store.setValue(PAR_AREA_ACCESS_SIZE + i, ParameterArea.parseAccessTypeInt(accessSize));
				} else {
					store.setToDefault(PAR_AREA_ACCESS_SIZE + i);
				}
				
				String readWait = cpuProp.getProperty(PAR_AREA_READ_WAIT + i);
				if (readWait != null) {
					store.setValue(PAR_AREA_READ_WAIT + i, Integer.parseInt(readWait));	
				} else {
					store.setToDefault(PAR_AREA_READ_WAIT + i);
				}
				
				String writeWait = cpuProp.getProperty(PAR_AREA_WRITE_WAIT + i);
				if (writeWait != null) {
					store.setValue(PAR_AREA_WRITE_WAIT + i, Integer.parseInt(writeWait));	
				} else {
					store.setToDefault(PAR_AREA_WRITE_WAIT + i);
				}
				
				
				String bigEndian = cpuProp.getProperty(PAR_BIG_ENDIAN + i);
				if (bigEndian != null) {
					store.setValue(PAR_BIG_ENDIAN + i, Boolean.valueOf(bigEndian).booleanValue());	
				} else {
					store.setToDefault(PAR_BIG_ENDIAN + i);
				}
				
			}
				
			String areaComment = cpuProp.getProperty(PAR_AREA_COMMENT + i);
			if (areaComment != null) {
				store.setValue(PAR_AREA_COMMENT + i, areaComment);	
			} else {
				store.setToDefault(PAR_AREA_COMMENT + i);
			}
			
		}
		
	}
	
	/**
	 * Reads the Debugger settings from the properties file and stores it into the given IPreferenceStore.
	 * @param cpuProp	the properties file to read from
	 * @param store	the IPreferenceStore to write to
	 */
	private void setCmdSettingsToStore(Properties cpuProp, IPreferenceStore store) {
		String debuggerType = cpuProp.getProperty(DEBUGGER_TYPE, DebuggerSetting.parseDebuggerTypeString(DebuggerSetting.TYPE_DEFAULT));
		String debugRamAddr = cpuProp.getProperty(DEBUG_RAM_ADDR, Long.toHexString(DebuggerSetting.RAM_ADDR_DEFAULT));
		try {
			store.setValue(DEBUGGER_TYPE, DebuggerSetting.parseDebuggerTypeInt(debuggerType));
			store.setValue(DEBUG_RAM_ADDR, Long.valueOf(debugRamAddr, 16).longValue());
		} catch (Exception e) {
			store.setToDefault(DEBUGGER_TYPE);
			store.setToDefault(DEBUG_RAM_ADDR);
		}
	}
	
	/**
	 * Returns a LinkerScriptSetting default
	 * @param settingName
	 * @param vector		info for the vector section
	 * @return the default LinkerScriptSetting based on the IPreferenceStore
	 * 
	 * Created from LinkerScriptSetting.createDefault()
	 */
	public static LinkerScriptSetting createDefaultLinkerScriptSetting(String settingName, SectionData vector) {
		
		LinkerScriptSetting setting = new LinkerScriptSetting(settingName);
		
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		boolean auto = store.getBoolean("sections.autoupdate");
		long bssVma = store.getLong(BSS_VMA);
		long vectorAddress = store.getLong(VECTOR_ADDRESS);	
		
		// add sections in the following order
		// .bss/.comm/.data/(.vector)/.text/.rodata/.ctors/.dtors/.gcc_execpt_table
		// bss section
		SectionData bss = new SectionData(SectionData.ATTR_BSS, auto);
		bss.setVma(bssVma);
		bss.setVmaIsAddress(true);
		
		// comm section (auto update should be true)
		SectionData comm = new SectionData(SectionData.ATTR_COMM, true);
		comm.setVmaSectionName(bss.getSectionName());
		
		
		// create text section
		SectionData text = new SectionData(SectionData.ATTR_TEXT, auto);
		
		// add .vector(if present) before .text
		if (vector != null) {
			// VMA should be set beforehand
			text.setVmaSectionName(vector.getSectionName());
		} else {
			// set the .text section VMA
			text.setVma(vectorAddress);
			text.setVmaIsAddress(true);
		}
		
		// rodata section
		SectionData rodata = new SectionData(SectionData.ATTR_RODATA, auto);
		rodata.setVmaSectionName(text.getSectionName());
		rodata.setVmaIsAddress(false);
		
		// ctors section (auto update should be true)
		SectionData ctors = new SectionData(SectionData.ATTR_CTORS, true);
		ctors.setVmaSectionName(rodata.getSectionName());
		
		// dtors section (auto update should be true)
		SectionData dtors = new SectionData(SectionData.ATTR_DTORS, true);
		dtors.setVmaSectionName(ctors.getSectionName());	   

		// ctors section (auto update should be true)
		SectionData except = new SectionData(SectionData.ATTR_GCC_EXCEPTION, true);
		except.setVmaSectionName(dtors.getSectionName());
		
		// data section
		SectionData data = new SectionData(SectionData.ATTR_DATA, auto);
		data.setVmaSectionName(comm.getSectionName());
		data.setVmaIsAddress(false);
		data.setLMAequalsVMA(false);
		data.setLmaSectionName(except.getSectionName());
		data.setLmaIsAddress(false);	   
		
		if (vector != null) {
			setting.addSection(vector);
		}
		setting.addSection(text);
		setting.addSection(rodata);
		setting.addSection(bss);
		setting.addSection(comm);
		setting.addSection(ctors);
		setting.addSection(dtors);
		setting.addSection(except);
		setting.addSection(data);
		
		
		return setting;
	}
	
	/**
	 * Returns a ParameterSetting default
	 * @param settingName
	 * @return	the default ParameterSetting based on the IPreferenceStore
	 * 
	 * Created from ParameterArea.createDefault()
	 */
	public static ParameterSetting createDefaultParameterSetting(String settingName) {
		
		ParameterSetting setting = new ParameterSetting(settingName);
		
		// get the preference store with the area settings
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();

		// create areas
		int areaCount = store.getInt(ICpuPreferenceConstants.PAR_AREA_COUNT);
		ParameterArea[] areas = new ParameterArea[areaCount];
		for (int i = 0; i < areas.length; i++) {

			// create each area according to the store value
			areas[i] = new ParameterArea();
			
			int type = store.getInt(PAR_AREA_TYPE + i);
			areas[i].setAreaType(type);
			
			long start = store.getLong(PAR_AREA_START + i);
			areas[i].setStartAddress(start);
			
			areas[i].setEndAddress(start + store.getLong(PAR_AREA_SIZE + i) - 1);
			areas[i].setComment(store.getString(PAR_AREA_COMMENT + i));
			
			if (type != ParameterArea.AREA_STACK) {
				areas[i].setAccessType(store.getInt(PAR_AREA_ACCESS_SIZE + i));
				areas[i].setWaitCycleRead(store.getInt(PAR_AREA_READ_WAIT + i));
				areas[i].setWaitCycleWrite(store.getInt(PAR_AREA_WRITE_WAIT + i));
				areas[i].setUseBigEndian(store.getBoolean(PAR_BIG_ENDIAN + i));
			}
			
			setting.addArea(areas[i]);
		}
		
		return setting;
		
	}
	
	
	/**
	 * Returns a DebuggerSetting default
	 * @param settingName
	 * @return	the default DebuggerSetting based on the IPreferenceStore
	 * 
	 * Created from DebuggerSetting.DebuggerSetting()
	 */
	public static DebuggerSetting createDefaultDebuggerSetting(String settingName) {
		
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		//EfobK^Cvݒ(|[g͎I)
		DebuggerSetting setting = new DebuggerSetting();
		
		// set name and debugger type
		setting.setDefaultSettingName(settingName);
		setting.setDebuggerType(store.getInt(DEBUGGER_TYPE));
		
	
		// set TTBR(from wizard input)
		setting.setVectorAddress(store.getLong("wizard.vecAddr"));
		
		// debugRAM addr
		setting.setDebugRamAddr(store.getLong(DEBUG_RAM_ADDR));
		
		return setting;
	}
	
	
	/**
	 * Returns a map of GeneralInfo objects created from 
	 * the cpu files. 
	 * Must be called after initialize()
	 * @return Map of GeneralInfo
	 */
	public Map getCpuInfoMap() {
		return infoMap;
	}

	/**
	 * Returns the CPU name for the given display name 
	 * @param cpuDisplayName	the key for the CPU name
	 * @return	the CPU namee for the given display name
	 * 			returns null if not found 
	 * @since 1.4.1
	 */
	public String getCpuName(String cpuDisplayName) {
		String cpuName = null;
		
		String[] cpuNames = getCpuNames();
		for (int i = 0; i < cpuNames.length; i++) {
			GeneralInfo general = (GeneralInfo) infoMap.get(cpuNames[i]);
			// found
			if (general.getCpuDisplayName().equals(cpuDisplayName)) {
				cpuName = cpuNames[i]; 
				break;
			}
		}
		
		return cpuName;
	}

	/**
	 * Returns the CPU names in String[]
	 * @return a String array of CPU names 
	 */
	public String[] getCpuNames() {
		Set keySet = infoMap.keySet();
		return (String[]) keySet.toArray(new String[keySet.size()]);
	}

	/**
	 * Returns the display name for the given CPU name
	 * @param cpuName	the key for the display name
	 * @return	the display name for the given CPU name
	 * @since 1.4.1
	 */
	public String getCpuDisplayName(String cpuName) {
		
		GeneralInfo general = (GeneralInfo) infoMap.get(cpuName);
		return general.getCpuDisplayName();
	}

	
	/**
	 * Returns the CPU display names in String[]
	 * as the same order as getCpuNames()
	 * @return a String array of CPU names 
	 * @since 1.4.1
	 */
	public String[] getCpuDisplayNames() {
		String[] displayNames = null;
		
		String[] cpuNames = getCpuNames();
		displayNames = new String[cpuNames.length];
		
		for (int i = 0; i < cpuNames.length; i++) {
			displayNames[i] = getCpuDisplayName(cpuNames[i]);
		}
		
		return displayNames;
	}

	
	/**
	 * Returns the initial TTBR value for the given CPU name
	 * @param cpuName
	 * @return	the initial TTBR value
	 */
	public long getInitialTtbrValueForCpu(String cpuName) {
		GeneralInfo general = (GeneralInfo) infoMap.get(cpuName);
		if (general == null) {
			return 0;	 
		}
		return general.getInitialTtbrValue();
		
	}
	
	/**
	 * Tokenize the property value by separator and returns them as
	 * String[]
	 * @param value		to be split
	 * @param separator	1 char separator to split the value
	 * @return	the split values
	 */
	protected String[] splitValues(String value, String separator) {
		
		StringTokenizer tokenizer = new StringTokenizer(value, separator);
		int tokens = tokenizer.countTokens();
		String[] ret = new String[tokens];

		for (int i = 0; i < tokens; i++) {
			ret[i] = tokenizer.nextToken().trim();
		}
		
		return ret;
		
	}
	
	public boolean isInitialized() {
		return initialized;
	}
	
	/**
	 * Returns the CPU name from the given CPU type
	 * for project file compatibilities.
	 * 
	 * @param cpuType
	 * @return the new CPU name for the project file
	 */
	public static String newCpuNameFromType(int cpuType) {
		
		if (cpuType == GeneralInfo.CPU_STD) {
			return STD_NAME; 
		} else if (cpuType == GeneralInfo.CPU_PE) {
			return PE_NAME;
		} else if (cpuType == GeneralInfo.CPU_ADV) {
			return ADV_NAME;
		}

		return "unknown";		// should never occur
		
	} 
	
	/**
	 * Return the file list in File objects
	 * @return CPU setting files
	 */
	private File[] getFileList() {
		return cpuDir.listFiles(new CpuFileFilter());
	}
	
	
	/**
	 * Filter for the cpu files
	 */
	private class CpuFileFilter implements FilenameFilter {
		public boolean accept(File dir, String name) {
			if (name.startsWith(FILE_PREFIX) && name.endsWith(FILE_SUFFIX)) {
				return true;
			}
			return false;
		}
		
	}
	
	/**
	 * Special Comparator for the SortedMap
	 * 
	 * For compatibility reasons, STD and PE must come as first
	 * in the CPU list. 
	 * 
	 * @author df
	 * @since 1.4.0
	 */
	private class CpuNameComparator implements Comparator {

		public int compare(Object o1, Object o2) {
	
			String s1 = (String) o1;
			String s2 = (String) o2;
	
			// same
			if (s1.equals(s2)) {
				return 0;
			} 
			
			if (s1.equals(STD_NAME)) {
				return -1;			// STD always comes first
			} else if (s1.equals(PE_NAME)) {
				if (s2.equals(STD_NAME)) {
					return 1;		// PE comes second
				} else {
					return -1;
				}
			} else if (s2.equals(STD_NAME) || s2.equals(PE_NAME)) {
				return 1;			// STD or PE comes first
			}

			// regular comparison
			return s1.compareTo(s2);
		}
	}
	
}
