/*
 * Created on 2004/10/05
 * Modified on 2005/11/11 by D.Fujimoto added releaseStream()
 * 										modified createFile() to prevent clock skew in make.exe.
 * Modified on 2006/03/14 by D.Fujimoto refactored makeContents()
 * Modified on 2006/03/17 by D.Fujimoto added getFileName()
 * Modified on 2006/06/15 by D.Fujimoto added replaceEnvVars().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.IEnvVar;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;

/**
 * Abstract class for creating a file in the project 
 * during a project build.
 * @author D.Fujimoto
 *	
 */
public abstract class AbstractFileGenerator {

	public static final String NEW_LINE = "\n"; //$NON-NLS-1$ 
	public static final String TAB = "\t"; //$NON-NLS-1$
	public static final String SPACE = " "; //$NON-NLS-1$
	public static final String BACK_SLASH = "\\";	

	// file extensions
	public static final String EXT_MAK = ResourceProvider.getResourceString("makefile.extension");
	public static final String EXT_CMD = ResourceProvider.getResourceString("commandfile.extension");
	public static final String EXT_LNK = ResourceProvider.getResourceString("linkerscriptfile.extension");
	public static final String EXT_MAP = ResourceProvider.getResourceString("mapfile.extension");
	public static final String EXT_PAR = ResourceProvider.getResourceString("parameterfile.extension");
	public static final String EXT_EXE = ResourceProvider.getResourceString("executable.extension");
	public static final String EXT_OBJ = ResourceProvider.getResourceString("objectfile.extension");
	public static final String EXT_DMP = ResourceProvider.getResourceString("objdumpfile.extension");
	
	private static final String templatePath = "templates";
	
	public static final String ERROR_OPEN_FAILED = ResourceProvider.getResourceString("file.open.failed");

	/**
	 * The descriptor manager.
	 */
	protected Gnu33ProjectDescriptorManager manager;

	/**
	 * CPU settings.
	 */
	protected CpuSettingsManager cpuManager;

	/**
	 * The targetted project.
	 */
	protected IProject project;

	/**
	 * Handle for file creation.
	 */
	protected IFile file;
	/**
	 * Stream to be used for creating the file.
	 */
	protected InputStream inputStream;

	/**
	 * The CPU type of this project.
	 * @see jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo
	 */
	protected int cpuType;
	
	
	/**
	 * Constructor. Creates the file's handle which will be be created later on.
	 * 
	 * @param project
	 *			  the project in which the file will be created.
	 * @param manager
	 *			  the descriptor manager
	 * @param fileName
	 *			  the file name to be created
	 */
	public AbstractFileGenerator(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		this.project = project;
		this.manager = manager;
		file = project.getFile(fileName);
		cpuType = manager.getCPUType();
		cpuManager = CpuSettingsManager.getManager();
	}

	/**
	 * Set the template file into inputStream.
	 * 
	 * @param templateFileName -
	 *			  file name of template file
	 */
	public void openTemplateFileStream(String templateFileName) {
		try {
			URL templateFileLocation = FileLocator.find(Gnu33CorePlugin.getDefault().getBundle(), 
					new Path(templatePath).append(templateFileName), null);
			inputStream = templateFileLocation.openStream();
		} catch (Exception e) {
			Logger.log(ERROR_OPEN_FAILED + templatePath + IPath.SEPARATOR
					+ templateFileName);
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns the file name to be generated 
	 * @return file name
	 * @since 1.3.0
	 */
	public String getFileName() {
		return (file != null) ? file.getName() : null ;
	}

	/**
	 * Every subclass must implement this method. Makes the contents of the file
	 * herein.
	 * @return the contents of the generated file
	 */
	public abstract String makeContents();

	/**
	 * Converts the given contents to an InputStream. Should be called after
	 * makeContents().
	 * 
	 * @param contents
	 *			  the file's contents
	 */
	public void setContentsToStream(String contents) {

		inputStream = new ByteArrayInputStream(contents.getBytes());

	}
	
	/**
	 * Closes the stream when finished.
	 * This should be called after the builder no longer needs
	 * the files contents.
	 * @throws IOException
	 * @since 1.2.0
	 */
	public void releaseStream() throws IOException {
		if (inputStream != null) {
			inputStream.close();
		}
	}

	/**
	 * Creates the file into the project. If a file already exists, recreate it.
	 * 
	 * @param monitor
	 *			  ProgressMonitor
	 * @throws CoreException
	 *			   when file creation failed
	 * @throws InterruptedException
	 */
	public void createFile(IProgressMonitor monitor) throws CoreException {
  
		
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		monitor.beginTask("Creating " + file.getName(), 1);

		try {
			if (file.exists()) {
				file.delete(true, false, monitor);
			}
			
			file.create(inputStream, IResource.FORCE | IResource.DERIVED, monitor);

		} catch (CoreException e) {
			throw e;
		}
			
		if (monitor.isCanceled()) {
			throw new OperationCanceledException();
		}
		
		// refresh the file
		file.refreshLocal(IResource.DEPTH_ZERO, monitor);
		
		// dummy operations to prevent clock skew at make.exe
		try {
			
			// open and close the file
			InputStream dummyStream = file.getContents();
			if (dummyStream != null) {
				dummyStream.close();
			}
			
		} catch (IOException e) {
			Logger.log("Stream open error(" + file.getName() + ")");
		}
	   
		monitor.worked(1);
		
	}

	/**
	 * Replaces all the env variables Strings to their values 
	 * within the given file contents 
	 * 
	 * @param contents		contents to be converted
	 * @param envVars		List of environments variables
	 * @return	the converted contents
	 * @since 1.3.0
	 */
	public String replaceEnvVars(String contents, List envVars) {
		
		// the format is $(NAME) or ${NAME}
		for (int i = 0; i < envVars.size(); i++) {
			IEnvVar env = (IEnvVar) envVars.get(i);
			contents = contents.replaceAll("\\$\\(" + env.getName() + "\\)", env.getValue());
			contents = contents.replaceAll("\\$\\{" + env.getName() + "\\}", env.getValue());
		}
		return contents;
	
	}

}
