/*
 * Created on 2004/08/25
 * Modified on 2005/04/07 by D.Fujimoto added getAnsiLibPath()
 * 										added getAnsiLibName()
 * Modified on 2005/07/12 by D.Fujimoto added getLibPath() getLibPathByCpuType()
 * 										added getCppLibName() getLongLongLibName()
 * 										deleted getAnsiLibPath() and getEmulibPath()
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/06/29 by D.Fujimoto added getSimIOLibName()
 */
package jp.co.epson.ide.eclipse.gnu33.general;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/**
 * This class represents the project's general settings.
 * Settings in the descriptor file is stored here and this object is 
 * stored in Gnu33Descriptor.
 * @author D.Fujimoto
 */
public class GeneralInfo {
	
	/**
	 * CPU type constants.
	 */
	public static final int CPU_STD 	= 0;
	public static final int CPU_PE 	= 1;
	public static final int CPU_ADV	= 2;
	public static final String CPU_STD_STR 	= "STD";
	public static final String CPU_PE_STR 	= "PE";
	public static final String CPU_ADV_STR 	= "ADV";
	public static final String CPU_ADV_33401_STR = "S1C33401";

	/**
	 * Memory model constants
	 */
	public static final int MEM_32 = 0;		// default 32bit accessing
	public static final int MEM_DP = 1;		// use data pointer
	
	public static final String MEM_32_STR = "MEM_32";	
	public static final String MEM_DP_STR = "MEM_DP";
	
   	public static final long TTBR_DEFAULT = 0xC00000L;
	
	/**
	 * CPU type.
	 * CPU_STD | CPU_PE | CPU_ADV
	 */
	private int cpuType;
	
  	/**
	 * CPU name
	 * 
	 */
	private String cpuName;
	
	/**
	 * CPU display name
	 * @since 1.4.1
	 */
	private String cpuDisplayName;
	
	/**
	 * Flag whether to use high precision library or not.
	 */
	private boolean useHighPrecisionLib;

	/**
	 * The memory model type
	 */
	private int memoryModel;
	
	/**
	 * Setting configuration
	 */
	private String currentConfName;
	
	/**
	 * The initial TTBR value to be 
	 * associated with this CPU type
	 * Not to be written to the project file
	 */
	private long initialTtbrValue;

	/**
	 * Constructor.
	 * Memory model defaults to 32 bit
	 */
	public GeneralInfo() {
		super();
		cpuType = CPU_STD;
		cpuName = "unknown";
		memoryModel = MEM_32;
		currentConfName = "default";
		initialTtbrValue = TTBR_DEFAULT;
	}
	

	/**
	 * Parsses CPU type String to code.
	 * Used to read descriptor file.
	 * @param cpuType	the String
	 * @return 	the CPU type as code
	 */
	public static int parseCPUInt(String cpuType) {
		int ret = -1;
		if (cpuType.equals(CPU_STD_STR)) {
			ret = CPU_STD;
		} else if (cpuType.equals(CPU_PE_STR)) {
			ret = CPU_PE;
		} else if (cpuType.equals(CPU_ADV_STR) || cpuType.equals(CPU_ADV_33401_STR)) {
			ret = CPU_ADV;
		}
		return ret;
	}
	
	/**
	 * Parses CPU type String to code.
	 * Used to write descriptor file.
	 * @param cpuType	the code
	 * @return 	the CPU type as String
	 */
	public static String parseCPUString(int cpuType) {
		String ret = null; 
		switch (cpuType) {
			case CPU_STD :
				ret = CPU_STD_STR;
				break;
			case CPU_PE :
				ret = CPU_PE_STR;
				break;
			case CPU_ADV :
				ret = CPU_ADV_STR;
				break;
		}
		return ret;
	}
	
	/**
	 * Parses Memory model String to code.
	 * @param memString	the String
	 * @return				the type as code
	 */
	public static int parseMemModelInt(String memString) {
		int ret = MEM_32;					// default
		if (memString.equals(MEM_DP_STR)) {
			ret = MEM_DP;
		}
		return ret;
	}
	
	/**
	 * Parses Memory model type to String
	 * @param memModel	the memory model type
	 * @return			the type as String
	 */
	public static String parseMemModelString(int memModel) {
		String ret = null;		// default
		switch (memModel) {
			case MEM_DP :
				ret = MEM_DP_STR;
				break;
			default:
				ret = MEM_32_STR;
				break;
					
		}
		return ret;
	}
	
	/**
	 * Returns the library path as an IPath object.
	 * The path is a relative path from the tools directory.
	 * The path depends on the currently set cpu type. 
	 * @return relative path from the tools directory.
	 * @since 1.1.1<br> All libraries are contained in the directory 
	 * 	corresponding the the cpu type.
	 * 
	 */
	public IPath getLibPath() {
		
		return getLibPathByCpuType(this.cpuType);
   	
	}
	
	/**
	 * Returns an IPath which indicates the library directory depending on the given cpu type.
	 * The path is a relative path from the tools directory.
	 * @param cpuType
	 * @return path to the library directory(relative)
	 * @since 1.1.1
	 */
	public static IPath getLibPathByCpuType(int cpuType) {
		String pathString = "";
		switch (cpuType) {
			case CPU_STD :
				pathString = ResourceProvider.getResourceString("gnu33tools.libpath.std");
				break;
			case CPU_PE :
				pathString = ResourceProvider.getResourceString("gnu33tools.libpath.pe");
				break;
			case CPU_ADV :
				pathString = ResourceProvider.getResourceString("gnu33tools.libpath.33401");
				break;
				
		}
		
		return new Path(pathString); 	
		
	}

	/**
	 * Returns the ansilib library name.
	 * @return the ansi c library file name.
	 * @since 1.1.0
	 */
	public String getAnsiLibName() {
		return ResourceProvider.getResourceString("gnu33tools.ansilibName");
	}
	
	/**
	 * Returns the emulib library name depending on use high precision flag.
	 * @return the emulation library file name.
	 */
	public String getEmulibName() {
		if (isUseHighPrecisionLib()) {
			return ResourceProvider.getResourceString("gnu33tools.emulibNameP");
		} else {
			return ResourceProvider.getResourceString("gnu33tools.emulibName");
		}
	}

	/**
	 * Returns the cpp lib library name.
	 * @return	the cpp lib library name.
	 * @since 1.1.1
	 */
	public String getCppLibName() {
		return ResourceProvider.getResourceString("gnu33tools.cpplibName");
	}
	
	/**
	 * Returns the library name for long long.
	 * @return the library name for long long.
	 * @since 1.1.1
	 */
	public String getLongLongLibName() {
		return ResourceProvider.getResourceString("gnu33tools.longlonglibName");
	}
	
	/**
	 * Returns the simulated IO library name.
	 * @return the simulated IO library name
	 * @since 1.4.0
	 */
	public String getSimIOLibName() {
		return ResourceProvider.getResourceString("gnu33tools.simIOlibName");
	}
	
	// setters and getters
	public int getCpuType() {
		return cpuType;
	}
	public void setCpuType(int cpuType) {
		this.cpuType = cpuType;
	}
	public String getCpuName() {
		return cpuName;
	}
	public void setCpuName(String cpuName) {
		this.cpuName = cpuName;
	}
	public String getCpuDisplayName() {
		return cpuDisplayName;
	}
	public void setCpuDisplayName(String cpuDisplayName) {
		this.cpuDisplayName = cpuDisplayName;
	}
	public int getMemoryModel() {
		return memoryModel;
	}
	public void setMemoryModel(int memoryModel) {
		this.memoryModel = memoryModel;
	}
	public boolean isUseHighPrecisionLib() {
		return useHighPrecisionLib;
	}
	public void setUseHighPrecisionLib(boolean useHighPrecisionLib) {
		this.useHighPrecisionLib = useHighPrecisionLib;
	}
	public String getCurrentConfName() {
		return currentConfName;
	}
	public void setCurrentConfName(String currentConfName) {
		this.currentConfName = currentConfName;
	}
	public long getInitialTtbrValue() {
		return initialTtbrValue;
	}
	public void setInitialTtbrValue(long initialTtbrValue) {
		this.initialTtbrValue = initialTtbrValue;
	}
	
}
