/*
 * Created on 2004/08/03
 * Modified on 2005/10/11 by D.Fujimoto added setDefaultSettingName()
 * 										modified Constructor(String).
 * Modified on 2007/03/13 by D.Fujimoto deleted createDefault() (moved to 
 * 								CPUSettingsManager)
 */
package jp.co.epson.ide.eclipse.gnu33.parameter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;

/**
 * This class represents a Parameter Setting.
 * A Parameter Setting has a List of Parameter Areas.
 * Settings in the descriptor file is stored here and this object is 
 * stored in the descriptor file.
 * 
 * @author D.Fujimoto
 *
 */
public class ParameterSetting {

	public static final String ERROR_DEFAULT_SETTING = 
		ParameterSettingResources.getString("error.default.par");
	public static final String ERROR_ADD_AREA = ParameterSettingResources.getString("error.invalid.area");
	
	/**
	 * The name of this setting.
	 * Also used as the file name for the parameter file.
	 */
	private String name = new String();
	
	/**
	 * The comment of this setting.
	 */
	private String comment = new String();
	
	/**
	 * The ParameterAreas contained in this setting.
	 * Presented as a List and is sorted by the 
	 * address of each ParameterArea.
	 */  
	private List areaList = new ArrayList();  
	
	/**
	 * Creates a ParameterSetting with no name
	 */
	public ParameterSetting() {
		super();
	}

	/**
	 * Creates a ParameterSetting with a name
	 * @param name
	 */
	public ParameterSetting(String name) {
		setDefaultSettingName(name);
	}
  

	/**
	 * Add an area to the List
	 * After being added, the list will sort itself
	 * If a stack area is added, the existing stack area will be 
	 * replaced
	 * @param area ParameterArea object to add to the List.
	 * @exception IllegalArgumentException - the ParameterArea address intersects with each other
	 */
	public void addArea(ParameterArea area) throws IllegalArgumentException {

		for (Iterator iter = areaList.iterator(); iter.hasNext();) {
			ParameterArea areaInList = (ParameterArea) iter.next();
			if (area.compareTo(areaInList) == 0) {
				throw new IllegalArgumentException();
			}
		}

		areaList.add(area);
		Collections.sort(areaList);
	}
	
	
	/**
	 * Removes the area from List of the given index
	 * After being removed, the list will sort itself
	 * @param index indicates which area to remove
	 */
	public void removeArea(int index) {
		areaList.remove(index);
		Collections.sort(areaList);
	}

	// getters and setters
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = comment;
	}
	public List getAreaList() {
		return areaList;
	}
	
	/**
	 * 
	 * @param name		name for the setting.
	 * @since 1.2.0
	 */
	public void setDefaultSettingName(String name) {
		setName(name + ResourceProvider.getResourceString("common.suffix"));
	}
	
	/**
	 * Displays the String expression for this class.
	 */
	public String toString() {
		String str = name + ":";  //$NON-NLS-1$
		for (Iterator iter = areaList.iterator(); iter.hasNext();) {
			ParameterArea area = (ParameterArea) iter.next();
			str += area.toString() + ","; //$NON-NLS-1$
		}
		return str;
	}
	

}
