/*
 * Created on 2004/08/18
 * Modified on 2005/08/05 by D.Fujimoto deleted makeCommandLineForDataPointers()
 * 										modified displayAllOptions() for data pointer removal.
 * Modified on 2006/10/31 by D.Fujimoto for parsing default options
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;



import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator;

import org.eclipse.cdt.managedbuilder.ui.properties.MultiLineTextFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.widgets.Text;


/**
 * A FieldEditorPage for each build tool.
 * The page will be shown when a tool is selected in the TreeViewer.
 * @author D.Fujimoto
 *
 */
public class BuildSettingsToolPage extends BuildSettingsInputPage {

	private static final String LABEL_COMMAND = BuildSettingDialogResources.getString("build.tool.command.label");
	private static final String LABEL_ALL_OPTIONS = BuildSettingDialogResources.getString("build.tool.allOps.label");
	
	/**
	 * The tool for this page.
	 */
	private IBuildTool tool;
	
	/**
	 * This BuildSetting's setting name.
	 */
	private String settingName;
	
	/**
	 * The ID for the All Options Editor.
	 */
	private String allOpsId;
	/**
	 * The FieldEditor for All Options.
	 */
	MultiLineTextFieldEditor allOptionsFieldEditor;
	
	/**
	 * The FieldEditor for tool command.
	 */
	StringFieldEditor commandField;
		
	/**
	 * Creates a new page for the given tool.
	 * @param tool
	 */
	public BuildSettingsToolPage(IBuildTool tool, String settingName) {
		super();
		this.tool = tool;
		this.settingName = settingName;
		allOpsId = tool.getToolID() + ".allOptions";
	}
	
	
	/**
	 * Create the field editors for this page.
	 */
	protected void createFieldEditors() {
		// call parent implementation to obtain the PreferenceStore
		super.createFieldEditors();
		
		commandField = new StringFieldEditor(tool.getToolID(), LABEL_COMMAND, 
				getFieldEditorParent());
		commandField.setEmptyStringAllowed(false);
		commandField.setStringValue(tool.getCommand());
		Text textCommand = commandField.getTextControl(getFieldEditorParent());
		textCommand.setEditable(false);
		addField(commandField);
		
		allOptionsFieldEditor = new MultiLineTextFieldEditor(allOpsId, 
				LABEL_ALL_OPTIONS, getFieldEditorParent());
		Text textAllOps = allOptionsFieldEditor.getTextControl(getFieldEditorParent());
		textAllOps.setEditable(false);
		addField(allOptionsFieldEditor);
		
	}
	
	
	/**
	 * Creates the command line for this tool and show 
	 * it in the All Options field.
	 *
	 */
	public void displayAllOptions() {
		
		IPreferenceStore store = getPreferenceStore();
		
		// the command line will be stocked in here
		StringBuffer buf = new StringBuffer();

		// make the command line
		List categoryList = tool.getOptionCategories();
		for (Iterator iter = categoryList.iterator(); iter.hasNext();) {
			IBuildCategory category = (IBuildCategory) iter.next();
			List optionList = category.getOptions();
			IBuildOption[] options = (IBuildOption[]) optionList.toArray(new IBuildOption[0]);

			makeCommandLine(buf, options, store);
			
		}
		
		// append the default options
		String[] defOptions = tool.getDefaultOptions();
		StringBuffer defaultBuf = new StringBuffer();
		for (int i = 0; i < defOptions.length; i++) {
			defaultBuf.append(defOptions[i] + " "); 
		}
		
		
		// parsing assembler options
		if (tool.getToolID().equals(BuildSettingFactory.ASSEMBLER_ID)) {
			StringBuffer optBuf = new StringBuffer();
			
			// add -Wa,options,...
			if (buf.length() > 0) {
				optBuf.append("-Wa,");
				
				StringTokenizer tok = new StringTokenizer(new String(buf));
				while (tok.hasMoreTokens()) {
					optBuf.append(tok.nextToken());
					if (tok.hasMoreTokens()) {
						optBuf.append(",");
					}
				}
			}
			buf = optBuf;
		}
			
		// default options + regular options
		buf = defaultBuf.append(" ").append(buf);

		// show it
		store.setValue(allOpsId, new String(buf));
		allOptionsFieldEditor.load();

	}
	
	/**
	 * Parses the options into a command line.
	 * Almost the same as
	 * jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator#
	 * makeCommandLine(StringBuffer, IBuildOption[]).
	 * @param buf the command line is stuffed in here
	 * @param options the options to parse
	 * @param store the PreferenceStore in which the current settings are stored
	 */
	public void makeCommandLine(StringBuffer buf, IBuildOption[] options, IPreferenceStore store) {
		
		for (int i = 0; i < options.length; i++) {
			
			IBuildOption option = options[i];

			switch (option.getType()) {
				case IBuildOption.TYPE_BOOLEAN :
					if (store.getBoolean(option.getId())) {
						buf.append(option.getCommand() + IBuildTool.SPACE);
						if (option.getId().equals(BuildSettingFactory.GEN_MAP_OPT_ID)) {
							// for -Map option
							buf.append(settingName 
									+ MakeFileGenerator.EXT_MAP).append(IBuildTool.SPACE);
						}
					}
					break;
				case IBuildOption.TYPE_STRING :
				case IBuildOption.TYPE_STRING_MULTI :
					String val = store.getString(option.getId());
					if (val.length() > 0) {
						buf.append(val + IBuildTool.SPACE);
					}
					break;
				case IBuildOption.TYPE_ENUM :
					String selectedStr = store.getString(option.getId());
					String enumCmd = option.getEnumCommand(selectedStr);
					if (enumCmd != null) {
						buf.append(enumCmd + IBuildTool.SPACE);
					}						
					break;
				case IBuildOption.TYPE_LIST :
					// do not show the libraries
					if (option.getId().equals(BuildSettingFactory.LD_LIB_OPT_ID)) {
						break;
					}
					String listVal = store.getString(option.getId());
					String listCmd = option.getCommand();
					StringTokenizer tokenizer = new StringTokenizer(listVal, IBuildOption.LIST_DELIMITER);
					while (tokenizer.hasMoreTokens()) {
						String token = tokenizer.nextToken();
						if (token.length() > 0) {
							buf.append(listCmd + token + IBuildTool.SPACE);    
						}
					}
					break;
			}
		}
	}
	
	
	public boolean performOk() {
		// write values to the PreferenceStore
		 boolean ok = super.performOk();
		 
		 tool.setCommand(getPreferenceStore().getString(tool.getToolID()));
		 
		 return ok;
	}
	
	
}
