/*
 * Created on 2004/08/04
 * Modified on 2005/11/14 by D.Fujimoto added editedArea and its getter
 * 										modified okPressed().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0) for Eclipse3.2
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.parameter;

import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterArea;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSettingResources;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * This is the input dialog for editing a new ParameterArea.
 * @author D.Fujimoto
 *
 */
public class ParameterEditDialog extends TitleAreaDialog {

	public static final String MSG_INVALID_COMMENT = ParameterSettingDialogResources.getString("comment.input.invalid");
	
	// title and message of this dialog
	public static final String DLG_DESC = ParameterSettingDialogResources.getString("parameter.dlg.desc");
	public static final String DLG_MSG = ParameterSettingDialogResources.getString("parameter.dlg.msg");

	// width and height of this dialog
	private static final int DLG_WIDTH = 270;
	private static final int DLG_HEIGHT = 460;

	/**
	 * Add mode constant for setMode(). Also the dialog title.
	 */
	public static final int MODE_ADD = 1;

	/**
	 * Edit mode constant for setMode(). Also the dialog title.
	 */
	public static final int MODE_EDIT = 2;

	/**
	 * Mode for the dialog.
	 */
	private int mode = MODE_ADD;

	/**
	 * Contains the edited data for this dialog.
	 */
	private ParameterArea areaData = new ParameterArea();

	/**
	 * The setting list.
	 */
	private ParameterSetting setting;

	/**
	 * The edited area which should be selected in the
	 * TableViewer after this dialog closes.
	 */
	private ParameterArea editedArea;
	

	/**
	 * The ParameterSetting index that indicates which Area we are editing.
	 * Use only for edit mode.
	 */
	private int editIndex;

	/**
	 * The error message string.
	 */
	private String errorMessage;

	// the controls
	private Combo comboAreaType;
	private Text textStartAddr;
	private Text textEndAddr;
	private Text textWaitRead;
	private Text textWaitWrite;
	private Combo comboAccessSize;
	private Button checkBigEndian;
	private Text textComment;

	/**
	 * Initialize the dialog.
	 * @param parentShell parent window(dialog)
	 * @param setting ParameterSetting object this dialog will add an area to
	 */
	public ParameterEditDialog(Shell parentShell, ParameterSetting setting) {
		super(parentShell);
		this.setting = setting;
	}

	/**
	 * Sets an initial size of this dialog.
	 */
	protected Point getInitialSize() {
		Point shellSize = getShell().computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		return new Point(Math.max(	DLG_WIDTH, shellSize.x),
				Math.max(DLG_HEIGHT, shellSize.y));
	}

	/**
	 * Sets the title for the dialog.
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);

		// set the title here
		String title = null;
		if (mode == MODE_ADD) {
			title = ParameterSettingDialogResources.getString("parameter.dlg.title.add");
		} else if (mode == MODE_EDIT) {
			title = ParameterSettingDialogResources.getString("parameter.dlg.title.edit");
		}
		newShell.setText(title + ParameterSettingDialogResources.getString("parameter.dlg.title.area"));
	}


	/**
	 * Creates the controls for this dialog
	 *
	 * @param parent
	 *			  The parent Composite
	 */
	protected Control createDialogArea(Composite parent) {

		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());

		int alignStyle = GridData.HORIZONTAL_ALIGN_FILL;

		// type and address block
		{
			Composite typeAddrBlock = new Composite(container, SWT.NONE);
			GridLayout gridLayout = new GridLayout();
			gridLayout.makeColumnsEqualWidth = true;
			gridLayout.numColumns = 2;
			typeAddrBlock.setLayout(gridLayout);
			GridData gridData = new GridData(alignStyle);
			typeAddrBlock.setLayoutData(gridData);

			Label labelType = new Label(typeAddrBlock, SWT.NONE);
			labelType.setText(ParameterSettingDialogResources.getString("parameter.dlg.type.label"));

			comboAreaType = new Combo(typeAddrBlock, SWT.READ_ONLY);
			comboAreaType.setItems(new String[] {
					ParameterSettingResources.getString("ParEdit.label.ROM"),
					ParameterSettingResources.getString("ParEdit.label.RAM"),
					ParameterSettingResources.getString("ParEdit.label.IO"),
					ParameterSettingResources.getString("ParEdit.label.STACK")
					});
			comboAreaType.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					disableWhenStack();
					verifyInputs();
				}
			});
			GridData gridAreaType = new GridData();
			gridAreaType.widthHint = 78;
			comboAreaType.setLayoutData(gridAreaType);

			Label labelStartAddr = new Label(typeAddrBlock, SWT.NONE);
			labelStartAddr.setText(ParameterSettingDialogResources.getString("parameter.dlg.startAddr.label"));

			textStartAddr = new Text(typeAddrBlock, SWT.BORDER);
			textStartAddr.setTextLimit(UIUtil.getAddressMaxChars());
			textStartAddr.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableHexInput(e);
				}
			});
			textStartAddr.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					verifyInputs();
				}
			});
			GridData gridStartAddr = new GridData();
			gridStartAddr.widthHint = 100;
			textStartAddr.setLayoutData(gridStartAddr);

			Label labelEndAddr = new Label(typeAddrBlock, SWT.NONE);
			labelEndAddr.setText(ParameterSettingDialogResources.getString("parameter.dlg.endAddr.label"));

			textEndAddr = new Text(typeAddrBlock, SWT.BORDER);
			textEndAddr.setTextLimit(UIUtil.getAddressMaxChars());
			textEndAddr.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableHexInput(e);
				}
			});
			textEndAddr.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					verifyInputs();
				}
			});
			GridData gridEndAddr = new GridData();
			gridEndAddr.widthHint = 100;
			textEndAddr.setLayoutData(gridEndAddr);
		}


		// other infos block
		{
			Composite areaInfoBlock = new Composite(container, SWT.NONE);
			GridLayout gridLayout = new GridLayout();
			gridLayout.makeColumnsEqualWidth = true;
			gridLayout.numColumns = 2;
			areaInfoBlock.setLayout(gridLayout);
			GridData gridData = new GridData(alignStyle);
			areaInfoBlock.setLayoutData(gridData);

			Group groupAccessWait = new Group(areaInfoBlock, SWT.NONE);
			groupAccessWait.setText(ParameterSettingDialogResources.getString("parameter.dlg.waitGroup.label"));
			GridLayout layoutGroup = new GridLayout();
			layoutGroup.numColumns = 5;
			groupAccessWait.setLayout(layoutGroup);

			GridData gridAccessWait = new GridData(alignStyle);
			gridAccessWait.horizontalSpan = 2;
			gridAccessWait.heightHint = 42;
			gridAccessWait.widthHint = 224;
			groupAccessWait.setLayoutData(gridAccessWait);

			// Access Wait
			Label labelWaitRead = new Label(groupAccessWait, SWT.NONE);
			GridData gridLabelRead = new GridData();
			gridLabelRead.widthHint = 40;
			labelWaitRead.setLayoutData(gridLabelRead);
			labelWaitRead.setText(ParameterSettingDialogResources.getString("parameter.dlg.waitRead.label"));

			textWaitRead = new Text(groupAccessWait, SWT.BORDER);
			textWaitRead.setTextLimit(1);
			textWaitRead.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableHexInput(e);
				}
			});
			textWaitRead.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e){
					verifyInputs();
				}
			});
			GridData gridWaitRead = new GridData();
			gridWaitRead.widthHint = 30;
			textWaitRead.setLayoutData(gridWaitRead);

			Composite dummy = new Composite(groupAccessWait, SWT.NONE);
			GridData gridDummy = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
			gridDummy.widthHint = 23;
			gridDummy.heightHint = 30;
			dummy.setLayoutData(gridDummy);
			dummy.setLayout(new GridLayout());

			Label labelWaitWrite = new Label(groupAccessWait, SWT.NONE);
			GridData gridLabelWrite = new GridData();
			gridLabelWrite.widthHint = 40;
			labelWaitWrite.setLayoutData(gridLabelWrite);
			labelWaitWrite.setText(ParameterSettingDialogResources.getString("parameter.dlg.waitWrite.label"));

			textWaitWrite = new Text(groupAccessWait, SWT.BORDER);
			textWaitWrite.setTextLimit(1);
			textWaitWrite.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableHexInput(e);
				}
			});
			textWaitWrite.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					verifyInputs();
				}
			});
			GridData gridWaitWrite = new GridData();
			gridWaitWrite.widthHint = 30;
			textWaitWrite.setLayoutData(gridWaitWrite);

			// Access Size
			Composite accessSizeBlock = new Composite(areaInfoBlock, SWT.NONE);
			GridLayout gridLayout2 = new GridLayout();
			gridLayout2.makeColumnsEqualWidth = true;
			gridLayout2.numColumns = 2;
			accessSizeBlock.setLayout(gridLayout2);
			GridData gridData2 = new GridData(alignStyle);
			gridData2.horizontalSpan = 2;
			accessSizeBlock.setLayoutData(gridData2);



			Label labelAccessSize = new Label(accessSizeBlock, SWT.NONE);
			labelAccessSize.setText(ParameterSettingDialogResources.getString("parameter.dlg.acccessSize.label"));

			comboAccessSize = new Combo(accessSizeBlock, SWT.READ_ONLY);
			comboAccessSize.setItems(new String[] {
					ParameterSettingResources.getString("ParEdit.label.byte"),
					ParameterSettingResources.getString("ParEdit.label.halfword"),
					ParameterSettingResources.getString("ParEdit.label.word")
					});
			GridData gridAccessSize = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
			gridAccessSize.widthHint = 70;
			comboAccessSize.setLayoutData(gridAccessSize);


			// Use Big Endian
			checkBigEndian = new Button(areaInfoBlock, SWT.CHECK);
			GridData gridBigEndian = new GridData(GridData.FILL_HORIZONTAL);
			gridBigEndian.horizontalSpan = 2;
			checkBigEndian.setLayoutData(gridBigEndian);
			checkBigEndian.setText(ParameterSettingDialogResources.getString("parameter.dlg.bigEndian.label"));

			// Area Comment
			Label labelComment = new Label(areaInfoBlock, SWT.NONE);
			GridData gridLabelComment = new GridData();
			gridLabelComment.verticalSpan = 2;
			labelComment.setLayoutData(gridLabelComment);
			labelComment.setText(ParameterSettingDialogResources.getString("parameter.dlg.areaComment.label"));

			textComment = new Text(areaInfoBlock, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.WRAP);
			textComment.setTextLimit(100);
			GridData gridComment = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
			gridComment.heightHint = 60;
			gridComment.widthHint = 230;
			gridComment.horizontalSpan = 2;
			textComment.setLayoutData(gridComment);
			textComment.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					UIUtil.enableAsciiInput(e);
				}
			});
			textComment.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					verifyInputs();
				}
			});


		}

		// the title and the message
		setTitle(DLG_DESC);
		setMessage(DLG_MSG, IMessageProvider.NONE);

		//
		populateSetting();
		disableWhenStack();

		return container;
	}

	/**
	 * Fills in the controls with the set ParameterArea object.
	 */
	private void populateSetting() {
		// set the area type Combo
		comboAreaType.select(areaData.getAreaType());

		// set start and end address
		textStartAddr.setText(UIUtil.getAddressString(areaData.getStartAddress()));
		textEndAddr.setText(UIUtil.getAddressString(areaData.getEndAddress()));

		// set wait read/write
		textWaitRead.setText(areaData.getWaitCycleString(areaData.getWaitCycleRead()));
		textWaitWrite.setText(areaData.getWaitCycleString(areaData.getWaitCycleWrite()));

		// set the access size Combo
		comboAccessSize.select(areaData.getAccessType());

		// set the use bigendian check
		checkBigEndian.setSelection(areaData.isUseBigEndian());

		// set the area comment
		textComment.setText(areaData.getComment());

	}

	/**
	 * Disables access waits, access size, and bigendian checkbox when
	 * the area type is selected as STACK
	 */
	private void disableWhenStack() {
		boolean isStack = comboAreaType.getSelectionIndex() == ParameterArea.AREA_STACK;
		Control[] controls = {textWaitRead, textWaitWrite, comboAccessSize, checkBigEndian};

		for (int i = 0; i < controls.length; i++) {
			controls[i].setEnabled(!isStack);
		}
		controls = null;

	}

	/**
	 * Input checks for Text fields.
	 * Triggered by any text modifying event.
	 */
	private void verifyInputs() {
		errorMessage = null;
		long startAddr = 0;
		long endAddr = 0;

		try {
			// Address
			String startAddrStr = textStartAddr.getText();
			String endAddrStr = textEndAddr.getText();

			if (startAddrStr.length() != 0 && endAddrStr.length() != 0) {
				startAddr = Long.parseLong(startAddrStr, 16);
				endAddr = Long.parseLong(endAddrStr, 16);
				if (startAddr > endAddr) {
					errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.addrRange");
				}
			}
			if (startAddrStr.length() == 0 || endAddrStr.length() == 0) {
				errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.addrBlank");
			}

			// Wait cycle
			if (comboAreaType.getSelectionIndex() != ParameterArea.AREA_STACK) {
				if (textWaitRead.getText().length() == 0 || textWaitWrite.getText().length() == 0) {
					errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.waitBlank");
				}
			}
			
			// comment
			String comment = textComment.getText();
			if (!UIUtil.isAscii(comment)) {
				errorMessage = MSG_INVALID_COMMENT;
			}
			
		} catch (NumberFormatException ignore) {
			errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.addrNotHex");
		}

		// disable the OK button on errors
		if (errorMessage == null) {
			setMessage(DLG_MSG, IMessageProvider.NONE);
		} else {
			setMessage(errorMessage, IMessageProvider.ERROR);
		}
		Button ok = getButton(IDialogConstants.OK_ID);
		if (ok != null) {
			ok.setEnabled(errorMessage == null);
		}
	}


	/**
	 * Triggered when OK button is pressed.
	 * Adds the edited area information to the ParameterSetting object.
	 * If an area can not be added, an IllegalArgumentException will occur
	 * and sets an error message to the dialog.
	 */
	protected void okPressed() {

		// gather the input values
		ParameterArea newArea = new ParameterArea();
		newArea.setAreaType(comboAreaType.getSelectionIndex());
		newArea.setStartAddress(Long.parseLong(textStartAddr.getText(), 16));
		newArea.setEndAddress(Long.parseLong(textEndAddr.getText(), 16));
		if (!newArea.isStackArea()) {
			newArea.setWaitCycleRead(Integer.parseInt(textWaitRead.getText(), 16));
			newArea.setWaitCycleWrite(Integer.parseInt(textWaitWrite.getText(), 16));
			newArea.setAccessType(comboAccessSize.getSelectionIndex());
			newArea.setUseBigEndian(checkBigEndian.getSelection());
		}
		newArea.setComment(textComment.getText());

		if (mode == MODE_ADD) {
			try {
				// add the new area to the list
				setting.addArea(newArea);
			} catch (IllegalArgumentException e) {
				errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.overlap");
				setErrorMessage(errorMessage);
				// disable OK button
				Button ok = getButton(IDialogConstants.OK_ID);
				if (ok != null) {
					ok.setEnabled(errorMessage == null);
				}
				return;		// don't close the window

			}
		} else if (mode == MODE_EDIT) {
			// save the old area
			ParameterArea savedArea = new ParameterArea();
			savedArea.setAreaType(areaData.getAreaType());
			savedArea.setStartAddress(areaData.getStartAddress());
			savedArea.setEndAddress(areaData.getEndAddress());
			savedArea.setAccessType(areaData.getAccessType());
			savedArea.setWaitCycleRead(areaData.getWaitCycleRead());
			savedArea.setWaitCycleWrite(areaData.getWaitCycleWrite());
			savedArea.setUseBigEndian(areaData.isUseBigEndian());
			savedArea.setComment(areaData.getComment());

			try{
				// remove the old area and
				// add the new area to the list
				setting.removeArea(editIndex);
				setting.addArea(newArea);
			} catch(IllegalArgumentException e) {
				// revert the saved area
				setting.addArea(savedArea);
				errorMessage = ParameterSettingDialogResources.getString("parameter.dlg.msg.overlap");
				setErrorMessage(errorMessage);
				// disable OK button
				Button ok = getButton(IDialogConstants.OK_ID);
				if (ok != null) {
					ok.setEnabled(errorMessage == null);
				}
				return;		// don't close the window
			}
		}
		
		editedArea = newArea;

		// Return OK code
		super.okPressed();

	}


	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Sets the open mode of this dialog.
	 * Modes are MODE_ADD | MODE_EDIT
	 * @param mode
	 */
	public void setMode(int mode) {
		this.mode = mode;
	}

	/**
	 * Sets the index of ParameterArea to edit.
	 * This is used only in EDIT mode.
	 *
	 * @param editIndex
	 */
	public void setEditIndex(int editIndex) {
		this.editIndex = editIndex;
		areaData = (ParameterArea) setting.getAreaList().get(editIndex);
	}

	/**
	 * Returns the ParameterArea that was edited in this dialog.
	 * @return the edited ParameterArea
	 * @since 1.2.0
	 */
	public ParameterArea getEditedArea() {
		return editedArea;
	}
}
