package jp.co.epson.ide.eclipse.gnu33.osmonitor.core;

import gnu33dll.OSMonitorConnection;

import java.io.File;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.ControllerFactory;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSInfo;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.OSInfoManager;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;


/**
 * The main plugin class to be used in the desktop.
 */
public class OSMonitorPlugin extends AbstractUIPlugin {

	public static final String PLUGIN_ID = "jp.co.epson.ide.eclipse.gnu33.osmonitor"; //$NON-NLS-1$
	public static final String RESOURCE_BUNDLE = PLUGIN_ID + ".core.OSMonitorPluginResources"; //$NON-NLS-1$
	
	/**
	 * Relative path to the tools path from \eclipse directory
	 */
	private static String toolsRelativePath;
	
	/**
	 * The shared instance.
	 */
	private static OSMonitorPlugin plugin;
	
	
	/**
	 * The resource bundle. 
	 */
	private ResourceBundle resourceBundle;
	
	/**
	 * The currently connected OS.
	 * The OS manager will always exist. The loaded state
	 * will change in each session.
	 * 
	 */
	private OSInfoManager osManager;
	
	/**
	 * The monitor controller in use. 
	 * The controller will be created when the OS is confirmed as loaded.
	 */
	private IOSMonitorController controller;
	
	
	/**
	 * The constructor.
	 */
	public OSMonitorPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle(RESOURCE_BUNDLE);
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}
	

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);

		// get the toolsPathString
		toolsRelativePath = getResourceString("tools.relativePath");
		
		// create the OS manager
		osManager = new OSInfoManager();
		
	}
	
	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}
	
	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	/**
	 * Returns the shared instance.
	 */
	public static OSMonitorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle =getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}
	
	/**
	 * Returns the Eclipse Platform install path.
	 * @return install path as IPath.(e.g."c:/eclipse/")
	 */
	public static IPath getPlatformInstallPath() {
		
		Location loc = Platform.getInstallLocation();
		URL url = loc.getURL();
		File file = new File(url.getFile());
		String absPath = file.getAbsolutePath();
		IPath installPath = new Path(absPath);
		
		return installPath;
	}
	
	/**
	 * Returns an IPath(Absolute path) to the GNU Tools' directory.
	 * Uses relative path from eclipse.exe as a hint. 
	 * @return IPath to the GNU Tools' directory
	 */
	public static IPath getToolsPath() {
		IPath toolsPath = getPlatformInstallPath()
			.append(toolsRelativePath); //$NON-NLS-1$
		return toolsPath;
	}

	/**
	 * Returns the OS manager.
	 * @return OSInfoManager
	 */
	public OSInfoManager getOSManager() {
		return osManager;
	}
	
	
	/**
	 * Provides access to the monitor controller.<br>
	 * Should be invoked after prepareMonitorController(), other wise
	 * the result may be null.
	 * 
	 * @return IOSMonitorController
	 */
	public IOSMonitorController getMonitorController() {
		return controller;
	}
	

	/**
	 * Entry point for retreiving data from the connection.<br>
	 * 
	 * Reads the OS info and creates a controller based on the 
	 * OS name.
	 * If the OS cannot be read, the controller gets invalid.
	 * If the controller is invalid, re-creates the controller.
	 * If the controller is valid and the force flag is true, 
	 * 		initializes the controller again.
	 * If the controller is valid and the force flag is false, 
	 * 		does nothing.
	 * If the debugger is currently running, does none of the above
	 * and the OS/controller state will remain the same.
	 * 
	 * @param force	when set to true, re-initializes the controller
	 * 		(if one is available).
	 */
	public void prepareController(boolean force) {
		
		// do nothing if the target is running
		if (isConnectionBusy()) {
			return;
		}
		
		// get the session key
		long sessionKey = getSessionKey();
		
		// connect to debugger and get the OS info
		osManager.readOSInfo();
		
		if (osManager.isOsRead()) {
			// create a controller if it has not been created yet

			IOSInfo osInfo = osManager.getOSInfo();
			String osName = osInfo.getName();

			if (controller == null || !controller.isInitialized()) {
				// create a controller with the given OS name(first time or after an error)
				controller = ControllerFactory.createController(osName);

				if (controller != null) {
					controller.init(sessionKey);
				}

			} else {
				// session is continued and controller is active
				// even if the force flag is true, the session key may have changed
				if (force || controller.getSessionKey() != sessionKey) {
					controller.init(sessionKey);
				}
			}

			return;

		}
		
		// invalidate the controller 
		// if the OS is not read or if controller is not initialized
		invalidateController();
	
	}

	/**
	 * Invaildates the monitor controller.
	 * Meaning that one session is over and the controller should be 
	 * re-created and re-initialized after invoking this method.
	 * Does nothing if the controller is null.
	 *
	 */
	public void invalidateController() {
		if (controller != null) {
			controller.invalidate();
		}
	}
	
	/**
	 * Returns the busy state of the connection.
	 * @return
	 */
	public boolean isConnectionBusy() {
		return OSMonitorConnection.getConnection().isConnectionBusy();
	}
	
	/**
	 * Returns whether the state of connection is ready and readable. 
	 * @return
	 */
	public boolean isConnectionOk() {
		return OSMonitorConnection.getConnection().isConnectionOk();
	}
	
	/**
	 * Returns the value of the session key.
	 * @return
	 */
	public long getSessionKey() {
		return OSMonitorConnection.getConnection().sessionKey();
	}
	
	
	/**
	 * Deletes the log files in the plugin state directory.
	 * Throws no exceptions.
	 * @since 1.1.0
	 */
	public void deleteLogFiles() {
		
		IPath logPath = getStateLocation();
		File[] logFiles = logPath.toFile().listFiles();
		for (int i = 0; i < logFiles.length; i++) {
			logFiles[i].delete();
		}
	
	}

	
}
