/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for task info.
 * 
 * TaskData is used to set running/ready tasks for
 * TaskManagementData, and 
 * compute queues for any kernel object.  
 * 
 * @author D.Fujimoto
 *
 */
public class TaskData extends AbstractDataModel implements IQueue , INamedProcess {
	
	// status values
	public static final int STATUS_DMT = 0x0;
	public static final int STATUS_RUNNABLE = 0x1;	// running or ready
	public static final int STATUS_WAIT = 0x2;
	public static final int STATUS_SUS = 0x4;
	public static final int STATUS_WAS = 0x6;
	
	// sizes
	public static final int STATUS_LENGTH = 1;
	public static final int CUR_PRI_LENGTH = 1;
	public static final int EXC_PTN_LENGTH = 4;
	
	
	/**
	 * The name of this task.
	 */
	private String taskName;
	
	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;	
	
	/**
	 * Holds the status of this task. 
	 */
	private int taskStatus;
	
	/**
	 * Holds the current priority of this task.
	 */
	private int priority;
	
	/**
	 * Start request count.
	 */
	private int startReq;
	
	/**
	 * Wake up request count.
	 */
	private int wupReq;
	
	/**
	 * Address to where wait info is contained.
	 */
	private long waitInfoAddress;
	
	
	/**
	 * Stack pointer.
	 */
	private long spAddress;
	
	/**
	 * Program counter.
	 */
	private long pcAddress;

	// init info
	/**
	 * Initial Program counter.
	 */
	private long initPcAddress;
	
	/**
	 * Holds the initial priority of this task.
	 */
	private int initPriority;
	
	/**
	 * Initial Stack pointer.
	 */
	private long initSpAddress;
	
	/**
	 * Indices for queue.
	 */
	private int[] taskQueue;
	
	/**
	 * Constructor.
 	 * @param id
	 */
	public TaskData(int id) {
		super(id);
	}

	public long getInitPcAddress() {
		return initPcAddress;
	}
	public int getInitPriority(int minPriority) {
		return initPriority + minPriority;
	}
	public long getInitSpAddress() {
		return initSpAddress;
	}
	public long getPcAddress() {
		return pcAddress;
	}
	public int getPriority(int minPriority) {
		return priority + minPriority;
	}
	public long getSpAddress() {
		return spAddress;
	}
	public int getStartReq() {
		return startReq;
	}
	public int getTaskStatus() {
		return taskStatus;
	}
	public int getWupReq() {
		return wupReq;
	}
	public int getSuspendReq() {
		return (taskStatus & STATUS_SUS) != 0 ? 1 : 0;
	}
	public long getWaitInfoAddress() {
		return waitInfoAddress;
	}
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			// task control block
			case ToppersObjectTable.TYPE_TASK_CTRL :
				
				// QUEUE next
				buf.get(data);
				nextQueue = DataParser.parse(data);
				
				// QUEUE prev
				buf.get(data);
				previousQueue = DataParser.parse(data);
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
				
				// task status
				data = new char[STATUS_LENGTH];
				buf.get(data);
				taskStatus = ((int) DataParser.parse(data)) & 0x7;	// use only 0-2 bits
				
				// priority
				data = new char[CUR_PRI_LENGTH];
				buf.get(data);
				priority = (int) DataParser.parse(data);

				// IMPORTANT:check the bits for data from actcnt to enatex
				// read 1 byte
				data = new char[1];
				buf.get(data);
				int value = (int) DataParser.parse(data);
				
				// start request	0bit
				startReq = (value & 0x01);
				
				// wake up request	1bit
				wupReq = (value & 0x02) >>> 1;
				
				// enable exception	2bit(ignore)
			
				// SKIP 
				// unknown	(1)
				buf.get(data);
				
				// SKIP
				// TEXPTN	(4)
				data = new char[EXC_PTN_LENGTH];
				buf.get(data);
				
				// wait info address
				data = new char[FOUR_BYTES_LENGTH];
				buf.get(data);
				waitInfoAddress = DataParser.parse(data);
				
				// stack pointer
				buf.get(data);
				spAddress = DataParser.parse(data);
				
				// program counter
				buf.get(data);
				pcAddress = DataParser.parse(data);
				
				break;
			
			// task init block
			case ToppersObjectTable.TYPE_TASK_INIT :
				
				// tskatr	(ignore)
				buf.get(data);
				
				// exinf	(ignore)
				buf.get(data);
				
				// init program counter
				buf.get(data);
				initPcAddress = DataParser.parse(data);
				
				// init priority
				buf.get(data);
				initPriority = (int) DataParser.parse(data);
				
				// stksz	(ignore)
				buf.get(data);
				DataParser.parse(data);
				
				// init stack pointer
				buf.get(data);
				initSpAddress = DataParser.parse(data);
				
				
				break;
				
		
		}
		
	}
	

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue()
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue()
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		taskQueue = indices;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return taskQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getAddress()
	 */
	public long getAddress() {
		return initPcAddress;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getProcessName()
	 */
	public String getProcessName() {
		return taskName;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#setProcessName()
	 */
	public void setProcessName(String name) {
		taskName = name;
	}

}
